#!/usr/bin/env python3

import sys, argparse
from dypylib.bio.seq.base import GtfDict, check_overlap_e, check_overlap

def seperate_by_strand(genes):
    """ Seperate_genes by strand
    """
    pstv = [] # strand +
    ngtv = [] # strand -
    us = [] # strand ., unstrand
    for gene in genes:
        if gene.strand == "+":
            pstv.append(gene)
        elif gene.strand == "-":
            ngtv.append(gene)
        elif gene.strand == ".":
            us.append(gene)
        else:
            raise KeyError("undefind strand")
    return pstv, ngtv, us

def merge_overlap1(genes,pre="IHBMG",id_count=0):
    genes = sorted(genes, key = lambda x: x.start)
    curr_gene = genes[0]
    ifmerge = False
    for i in range(1,len(genes)):
        if check_overlap_e(curr_gene,genes[i]):
            curr_gene = curr_gene.merge(genes[i],pre+str(id_count))
            ifmerge = True
        # not overlap, so output.
        else:
            yield curr_gene
            curr_gene = genes[i]
            if ifmerge:
                id_count += 1
                ifmerge = False
    yield curr_gene
    id_count += 1
    #return id_count

def merge_genes(genes,new_id=None):
    curr_gene = genes[0]
    for i in range(1,len(genes)):
        curr_gene = curr_gene.merge(genes[i])
    if new_id:
        curr_gene.edit_gene_id(new_id)
    return curr_gene

def detect_fusion(data):
    """ input [(0, 1), (0, 2), (0, 3), (0, 4), (0, 5), (0, 6), (0, 7), (0, 8), (0, 9), (6, 7)]
        output [(0,(1,2,3,4))]
               [(index of fusion gene,index of member gene)]
    """ 

    def _get_over_dic(data):
        # val of over_dic: [index of overlap genes]
        count = {}
        def _add_count(a,b):
            count.setdefault(a,[]).append(b)
       
        for pair in data:
            _add_count(pair[0],pair[1])
            _add_count(pair[1],pair[0])
        return count
    
    def _get_max_count(over_dic):
        mymax = (0,[])
        for key, val in over_dic.items():
            #print("key")
            #print(key)
            #print("val")
            #print(val)
            if len(val) > len(mymax[1]):
               mymax = (key, val)
        return mymax

    def _del_e(data,name):
        outdata = []
        for pair in data:
            if name not in pair:
                outdata.append(pair)
        return outdata

    fusion = [] # Save the results.
    while 1:
        over_dic = _get_over_dic(data)
        mymax = _get_max_count(over_dic)
        if len(mymax[1]) < 2:
            return fusion
        else:
            fusion.append(mymax)
            data = _del_e(data,mymax[0])

def check_overlap_frac(st1,ed1,st2,ed2,thre1=0.6,thre2=0.6):
    if not check_overlap(st1,ed1,st2,ed2):
        return "N"
    overlap = min(ed1,ed2) - max(st1,st2) + 1
    frac1 = overlap/(ed1-st1+1)
    frac2 = overlap/(ed2-st2+1)
    if frac1 > thre1 and frac2 > thre2:
        return "OC"
    else:
        return "OU"

def merge_genes_in_block(genes):
    """ OU: two gene overlap, but not satisfied the threshold
        OC: two gene overlap and satisfied the threshold
        N: two gene not overlap
        results:
            gene cluster, fusion gene --- members.
            gene cluster:[[gene_1,gene_2],[gene_3],[gene_4,gene_5,gene6]]
            fusion:[(gene_1,(m_gene1,m_gene2)),(gene_2,(m_gene3,m_gene5,m_gene6))]

    """

    def _get_fusion_name(fusion):
        return (genes[fusion[0]].ID,tuple(map(lambda x:genes[x].ID,fusion[1])))

    def get_fusion_name(fusions):
        res = []
        for fu in fusions:
            res.append(_get_fusion_name(fu))
        return res

    def _update_st_ed(st1,ed1,st2,ed2):
        st = min(st1,st2)
        ed = max(ed1,ed2)
        return st, ed

    choosed = set() # The index of genes that have been collapsed.
    l = [] # lable record OU
    c = [] # collapsed genes
    for i in range(len(genes)):
        if i in choosed:
            continue
        choosed.add(i)
        curr_st = genes[i].start
        curr_ed = genes[i].end
        curr_collapsed = [i] # The index of genes OC.
        for j in range(len(genes)):
            if j in choosed:
                continue
            res = check_overlap_frac(curr_st,curr_ed,genes[j].start,genes[j].end)
            if res == "OU":
                l.append((i,j))
            elif res == "OC":
                curr_st, curr_ed = _update_st_ed(curr_st, curr_ed, genes[j].start, genes[j].end)
                curr_collapsed.append(j)
                choosed.add(j)
            elif res == "N":
                #c.append(curr_collapsed)
                break
        c.append(curr_collapsed)
    return map(lambda x:map(lambda y: genes[y],x),c), get_fusion_name(detect_fusion(l))

def iter_overlap_block(genes):
    """ Seperate whole annotation to different blocks by overlapping.
    """
    # Emplty block
    if not genes:
        return []
    genes = sorted(genes, key = lambda x: x.start)
    block = []
    curr_gene = genes[0]
    block.append(curr_gene)
    for i in range(1,len(genes)):
        if check_overlap_e(curr_gene,genes[i]):
            curr_gene = curr_gene.merge(genes[i])
            block.append(genes[i])
        # not overlap, so output.
        else:
            yield block
            curr_gene = genes[i]
            block = []
            block.append(curr_gene)
    yield block

def merge_and_write(genes,outfile,pre="IHBMG",id_count=0,ref_file=None):
    """ genes demo: [[gene_a,gene_b],[gene_c],[gene_d,gene_e,gene_f]]
        The first and the last list in demo list will be merged, then named after "pre" and id_count.
        output:
        xxx, pre_0(gene_a_b), xxx
        xxx, gene_c, xxx
        xxx, pre_1(gene_d_e_f), xxx
    """
    def _write_ref_file(genes,c_id,outfile,sep="\t"):
        if not outfile:
            return
        for gene in genes:
            outfile.write(gene.ID+sep+c_id+"\n")

    for sub_genes in genes:
        sub_genes = list(sub_genes) # convert map object to list.
        # collapse
        if len(sub_genes) > 1:
            c_id = pre+str(id_count)
            outfile.write(merge_genes(sub_genes,c_id).as_gtf()+"\n")
            id_count += 1
            _write_ref_file(sub_genes,c_id,ref_file)
        # output directly
        else:
            outfile.write(sub_genes[0].as_gtf()+"\n")
    return id_count

#def purish_fusion(ref_table,raw_fusion,out_fusion):
#    """ Read ref_table in to dict.
#        Repace fusion names with the dict and filter out those 
#    """
#    pass

def output_fusion(genes,outfile,sep="\t"):
    if not outfile:
        return
    for fu_mem in genes:
        for m in fu_mem[1]:
            outfile.write(fu_mem[0]+sep+m+"\n")

def merge_gtf(gtf,outfile,pre,ref_file,fusion_file):
    id_count = 0
    for genes in gtf.values():
        _genes = seperate_by_strand(genes.values())
        for strand in _genes:
            for block in iter_overlap_block(strand):
                gene_cluster, fusion_pairs = merge_genes_in_block(block)
                #### debug####
                #for i in gene_cluster:
                #    for j in i:
                #        print(j.ID)
                #### ########
                # merge and output
                id_count = merge_and_write(gene_cluster,outfile,id_count=id_count,ref_file=ref_file)
                # output fusion genes
                output_fusion(fusion_pairs,fusion_file)

def main(argv):
    parser = argparse.ArgumentParser(description="Merge overlaped gene.")
    parser.add_argument('gtf',nargs='?',help="input gtf file.",default = sys.stdin,type=GtfDict)
    parser.add_argument('-o','--outfile',nargs='?',help="output file",default=sys.stdout,type=argparse.FileType('w'))
    parser.add_argument('-g','--gene-id',nargs='?',default="IHBMG",help = "Prefix of gene id")
    parser.add_argument('-r','--refer-table',nargs='?',default=None,help ="Output tabular file to record old/new id",type=argparse.FileType('w'))
    parser.add_argument('-f','--fusion-gene',nargs='?',default=None,help ="Output fushion gene id and its target.",type=argparse.FileType('w'))
    args = parser.parse_args(argv[1:])

    merge_gtf(args.gtf,args.outfile,args.gene_id,args.refer_table,args.fusion_gene)

if __name__ == '__main__':
    main(sys.argv)



#!/usr/bin/env python3 

'''
_version=0.0.2
get subset of a fasta file randomly
'''

import sys
import random
from dypylib.bio.seq.base import *
#from bio.seq.base import Fasta, Falist, write_fasta_o

# overflow(true): If sample is large than population, return whole population.
def random_fa(seqs,outfile,output_num,overflow,lth):
    if overflow and len(seqs) < output_num:
        subset = seqs
    # return whole set when output num set to -1
    elif output_num == -1:
        subset = seqs
    else: 
        subset = random.sample(seqs,output_num)

    for seq in subset:
        write_fasta_o(outfile,seq,lth)

def random_fa_with_filter(seqs,keeps,outfile,output_num,overflow=False,filt_type="ref",lth=80,if_verse=False):
    """
    #ref_filt: read fasta head line in refSef style,">ref|refname|xxbd|xxname|" and only keep refname or xxname.
    if_verse: Set it to choose sequence outside keeps list rather than inside.
    """

    filt_seqs = []
    for seq in seqs:
        is_in = filter_keys(seq.name,keeps,filt_type)
        if (not if_verse) and (not is_in):
            continue
        if if_verse and is_in:
            continue
        #if seq.name in keeps:
        filt_seqs.append(seq)
    random_fa(filt_seqs,outfile,output_num,overflow,lth)

def read_keyf(key_file,col=0,sep="\t"):

    outdata = set()
    for line in key_file:
        tmp = line.strip().split(sep)
        if not tmp:
            continue
        outdata.add(tmp[col])
    return outdata

def filter_keys(myname,keeps,mytype="ref"):

    if mytype == "ref":
        return filter_ref_key(get_id_dic(myname),keeps)
    elif mytype == "GenCode":
        return filter_gcd_key(myname,keeps)
    elif mytype == "basic":
        return filter_bsc_key(myname,keeps)

def filter_ref_key(id_dic,keeps,key="ref"):

    myid = id_dic.get(key)
    if not myid:
        return False
    if myid in keeps:
        return True
    else:
        return False

def filter_gcd_key():

    pass

def filter_bsc_key(myname,keeps):

    if myname in keeps:
        return True
    else:
        return False

def get_id_dic(name,sep="|"):
    #get id information from refseq seq name

    tmp = name.split(sep)[:-1]
    key_val = {}
    for i in range(0,len(tmp),2):
        key_val[tmp[i]] = tmp[i+1]
    return key_val

def main(argv):

    import argparse

    parser = argparse.ArgumentParser(description="""
    Get subset of a fasta file randomly (for rand_num > 0)
            
    Example:
        1. Subset fasta file and keep sequence who's ID is in 
        the namelist.
            subset_fasta.py input.fa -k namelist -t basic
        2. Subset fasta file and keep sequence who's ID is not
        in the namelist.
            subset_fasta.py input.fa -k namelist -t basic -v
        3. Random select 100 sequences from fasta file.
            subset_fasta.py input.fa -n 100 -t basic
        4. Random select 100 sequences from fasta file and who's
        ID are in the namelist.
            subset_fasta.py input.fa -k namelist -n 100 -t basic
    """,
    formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('infile',nargs='?',help="fasta file for sampling ",default=sys.stdin,type=argparse.FileType('r'))
    parser.add_argument('-o','--outfile',nargs='?',help="output file",default=sys.stdout,type=argparse.FileType('w'))
    parser.add_argument('-k','--key_file',nargs='?',help="if key_file provided, only sequence in key_file should be keep",type=argparse.FileType('r'))
    parser.add_argument('-r','--if-trim',action='store_true',\
        help = ("Whether to trim fasta ID by [SPACE]? Set for yes.\n"
            "Note: this option will be replaced by --trim-name in "
            "the upcoming version."))
    parser.add_argument('--trim-name',help="Trim fasta ID by [SPACE]",action='store_true')
    parser.add_argument('-v','--if-verse',help="Choose sequences outside the key_file rather than inside the key_file",action='store_true')
    parser.add_argument('-t','--filt_type',nargs='?',help="filter type,can be \"ref\", \"basic\" or \"GenCode\"(not avaliable now). The default should be altered to basic.",default="ref")
    parser.add_argument('-l','--lowlim',nargs='?',help="low limits of length",type=int,default=-1)
    parser.add_argument('--overflow',dest='over',help="To allow sample be large than population(return whole fasta).",action='store_true')
    parser.add_argument('--row_len',nargs='?',help="sequence length of each row",type=int,default=80)
    parser.add_argument('-u','--uplim',nargs='?',help="up limits of length",type=int,default=float("inf"))
    parser.add_argument('-n','--rand_num',nargs='?',help="how many sequence should be random get",type=int,default=-1)
    args = parser.parse_args(argv[1:])

    if args.key_file:
        keys = read_keyf(args.key_file)
        #print(keys)
        myfa = Fasta(args.infile,l_lowlim=args.lowlim,l_uplim=args.uplim,if_trim=(args.if_trim or args.trim_name))
        #for key in myfa:
        #    print(key.name)
        #sys.exit()
        filt_type = args.filt_type
        random_fa_with_filter(myfa,keys,args.outfile,args.rand_num,args.over,filt_type,lth=args.row_len,if_verse = args.if_verse)#have option ref_filt
    else:
        myfa = Falist(args.infile,l_lowlim=args.lowlim,l_uplim=args.uplim,if_trim=(args.if_trim or args.trim_name)) 
        random_fa(myfa,args.outfile,args.rand_num,args.over,args.row_len)

if __name__ == '__main__':
    
    import sys
    sys.exit(main(sys.argv))



#!/usr/bin/env python3

'''
Add informations which are retrived from dbfile to infile.
'''

import sys
from dypylib.bio.base import DictFile

def extend_lst(lst, obj):
    """ Do not split string when extend.
        Why have to use this function: DictFile object return list
          when multiple element is chosen, while str when only one
          element is chosen.
    """
    if isinstance(obj, str):
        obj = [obj]
    return lst.extend(obj)

def mapper(infile,outfile,dbdata,sep,has_header=True,keycol=0,null=[""],value_header=None):

    #for output
    if not isinstance(null,list):
        null = [null]

    if has_header:
        header = infile.readline().rstrip("\n")
        # When infile and db file has header at the same time, output both header.
        # When header of db file is not exist, value_header can be appended.
        if dbdata.has_header:
            outfile.write(header+sep+sep.join(dbdata.header_lst)+"\n")
        elif value_header:
            outfile.write(header+sep+value_header+"\n")
        else:
            outfile.write(header+"\n")

    for eachline in infile:
        line_array = eachline.strip("\n").split(sep)
        values = dbdata.get(line_array[keycol],null*dbdata.valueLength())
        extend_lst(line_array, values)
        outfile.write(sep.join(line_array)+"\n")

def parse_value_col(value_col):

    if value_col == None:
        return value_col
    return map(int,value_col.split(","))

def main(argv):

    import argparse

    parser = argparse.ArgumentParser(description="Add information to file1 accoding to key-value lines in file2.")
    parser.add_argument('file1',nargs='?',help="file to be add information, \"-\" for stdin ")
    parser.add_argument('file2',nargs='?',help="dbfile, \"-\" for stdin")
    parser.add_argument('-o','--outfile',nargs='?',help="output file",default=sys.stdout,type=argparse.FileType('w'))
    parser.add_argument('-s1','--sep1',nargs='?',default="\t")
    parser.add_argument('-s2','--sep2',nargs='?',default="\t")
    parser.add_argument('-k1','--keycol1',nargs='?',default=0,type=int)
    parser.add_argument('-k2','--keycol2',nargs='?',default=0,type=int)
    parser.add_argument('-v','--valuecol',nargs='?',default=None,help="Format: 0,3,5,10. Whole line will be chose when not set this option")
    parser.add_argument('-p','--place_holder',nargs='?',default="",help="Set a place holder. For example \"-\" or \"NA\".")
    parser.add_argument('-vf','--value_header',nargs='?',default=None,help="Additional value header. Useful only when n1 set True and value length equals 1.")
    parser.add_argument('-n1','--has_header1',action='store_true',help="if file1 has header,set this option.")
    parser.add_argument('-n2','--has_header2',action='store_true',help='same with n1 but for file2')
    args = parser.parse_args(argv[1:])

    sep1=args.sep1
    sep2=args.sep2
    keycol1=args.keycol1#key column which define filter limits
    keycol2=args.keycol2
    has_header1=args.has_header1
    has_header2=args.has_header2
    valuecol=parse_value_col(args.valuecol)
    
    if args.file1 == '-' and args.file2 == '-':
        raise KeyError("You can not set file1 and file2 to \"-\" at the same time!")
    if args.file1 == '-':
        file1=sys.stdin
    else:
        file1=open(args.file1)
    if args.file2 == '-':
        file2=sys.stdin
    else:
        file2=open(args.file2)
    outfile=args.outfile

    dbdata = DictFile(args.file2,keypos=keycol2,delimiter=sep2,has_header=has_header2,valuepos=valuecol)
    mapper(file1,args.outfile,dbdata,sep1,has_header1,keycol1,null=args.place_holder,value_header=args.value_header)

if __name__ == '__main__':

    import sys
    main(sys.argv)



#!/usr/bin/env python

from ..base import Seq, Fasta


'''
Fasta format for Ensembl database.
Ensembl header:
>ENST00000347977 ncrna:miRNA chromosome:NCBI35:1:217347790:217347874:-1 gene:ENSG00000195671 gene_biotype:ncRNA transcript_biotype:ncRNA
   ^             ^     ^     ^                                          ^                    ^                           ^ 
   ID            |     |  LOCATION                            GENE: gene stable ID       GENE: gene biotype           TRANSCRIPT: transcript biotype   
                 |   STATUS
              SEQTYPE
Note: biotype seems to be ncrna not ncrna:miRNA.

Fasta format for refseq database.
NM_	mRNA	Protein-coding transcripts (usually curated)
NR_	RNA	Non-protein-coding transcripts
XM_c	mRNA	Predicted model protein-coding transcript
XR_c	RNA	Predicted model non-protein-coding transcript
https://www.ncbi.nlm.nih.gov/books/NBK21091/table/ch18.T.refseq_accession_numbers_and_mole/?report=objectonly
RefSeq header:
Case 1
>XM_010970720.1 PREDICTED: Camelus bactrianus transmembrane protein 11 (TMEM11), transcript variant X1, mRNA
Case 2
>XM_007508807.1 Bathycoccus prasinos PREDICTED: zinc finger protein 347-like (Bathy15g00730), partial mRNA
Case 3
>NM_001134752.1 Mus musculus small integral membrane protein 17 (Smim17), mRNA

    XM_010970720.1: accession id
    PREDICTED: status
    Camelus ... X1: Name
    mRNA: sequence type
'''
class EsbSeq(Seq):

    def __init__(self,*args):
    # args: string, name
        if args[0].__class__ == Seq:
            self.parse_name(args[0].name)
        else:
            super(EsbSeq,self).__init__(*args)
            self.parse_name(args[1])

    def parse_name(self,name):
        def get_value(string):
            tmp = string.split(":")
            return tmp[1]
        tmparray = name.strip().lstrip(">").split()
        self.ID = tmparray[0]
        self.seqtype = tmparray[1]
        self.location = tmparray[2]
        self.gene_id = get_value(tmparray[3])
        self.gene_type = get_value(tmparray[4])
        self.tx_type = get_value(tmparray[5])
        
class EsbFa(Fasta):

    def __iter__(self):

        self.fastaIter=self.iterFasta1()
        return self

    def iterFasta1(self):

        for seq in self.iterFasta():
            yield EsbSeq(seq)

class RefSeq(Seq):
    """
    Attributes:
        ID
        db
        species
        seq_name
    """

    def __init__(self,*args):
    # args: string, name
        if args[0].__class__ == Seq:
            self.parse_name(args[0].name)
        else:
            super(RefSeq,self).__init__(*args)
            self.parse_name(args[1])
        self.ID = self.ID.strip()

    def parse_name(self,name):
        tmparray = name.strip().lstrip(">").split("PREDICTED:")
        if len(tmparray) > 2:
            raise ValueError("%s has more than one \":\"" % ":".join(tmparray))
        elif len(tmparray) == 2:
            self._predicted_name(tmparray)
        else:
            self._normal_name(tmparray[0])
        self.db = self.ID.split("_")[0]

    def _predicted_name(self,tmparray):
        myID = tmparray[0].split()
        tmparray2 = tmparray[1].split()
        # case 2
        if len(myID)>1:
            self.ID = myID[0]
            self.species = " ".join(myID[1:])
            if len(myID) > 3:
                sys.stderr.write("Warning: %s is not particular case 2" % tmparray[0])
        # case 1
        else:
            self.ID = tmparray[0]
            self.species = " ".join(tmparray2[:2])
        self.seq_name = " ".join(tmparray2[2:])

    def _normal_name(self,name):
        tmparray = name.split()
        self.ID = tmparray[0]
        self.species = " ".join(tmparray[1:3])
        self.seq_name = " ".join(tmparray[3:])

class RefFa(Fasta):

    def __iter__(self):

        self.fastaIter=self.iterFasta1()
        return self

    def iterFasta1(self):

        for seq in self.iterFasta():
            yield RefSeq(seq)



#!/usr/bin/env python3

import sys, argparse
from dypylib.bio.seq.base import GtfDict

def rename_gtf(gtf, outfile, gene_id, tx_id, id_mapping=None):
    if id_mapping:
        genes, idmap = gtf.rename_gene_tx(gene_id, tx_id, get_id = True)
    else:
        genes = gtf.rename_gene_tx(gene_id, tx_id)
    for gid, gene in genes.items():
        for tid, tx in gene.items():
            for exon in tx.values():
                outfile.write(exon.as_str()+"\n")
    for idp in idmap:
        id_mapping.write("\t".join(idp)+"\n")

def main(argv):
    parser = argparse.ArgumentParser(description="Rename Gtf Gene id and tx id. (The groups are based on the old ID.)")
    parser.add_argument('gtf',nargs='?',help="input gtf file.",default = sys.stdin,type=GtfDict)
    parser.add_argument('-o','--outfile',nargs='?',help="output file",default=sys.stdout,type=argparse.FileType('w'))
    parser.add_argument('-g','--gene_id',nargs='?',default="HBG",help = "Prefix of gene id")
    parser.add_argument('-t','--tx_id',nargs='?',default=None,help = "Prefix of transcript id")
    parser.add_argument('-r','--refer-table',nargs='?',default=None,help ="Output tabular file to record old/new id",type=argparse.FileType('w'))
    args = parser.parse_args(argv[1:])

    rename_gtf(args.gtf,args.outfile,args.gene_id,args.tx_id,args.refer_table)

if __name__ == '__main__':
    main(sys.argv)



#!/usr/bin/env python3

import os, re
import pandas as pd

import sys
"""
    APIs:
    read_Tbout_raw
    read_Tbout
    read_Rfam_family
    read_spe_exp (deprecated)
    read_spe_exps (deprecated)
    read_ioe_file (deprecated): ioe file of suppa: hq_lq_lnc_b_all_strict.event.ioe
"""

def _tbout_get_header(src, comments='#'):
    """
        Estimate the number of field of tboutput from description lines.
        Assume the following format.
        example of header for hmmscan tboutput.
        #                                                                        --- full sequence ---- --- best 1 domain ---- --- domain number estimation ----
        # target name        accession  query name                    accession    E-value  score  bias   E-value  score  bias   exp reg clu  ov env dom rep inc description of target
        #------------------- ----------          -------------------- ---------- --------- ------ ----- --------- ------ -----   --- --- --- --- --- --- --- --- ---------------------
        :param:src: file handle.
    """
    sep = re.compile(r'-\s')
    theHeader = 2
    theNumber = 3
    counts = 0
    out_header = []
    for line in src:
        line = line.strip()
        if line[0] == comments:
            line = line.lstrip(comments)
            counts += 1
        if counts == theHeader:
            header = line
        if counts == theNumber:
            # Get index according to the third line.
            idx = [0]
            idx.extend([i.end() for i in sep.finditer(line)])
            for i in range(len(idx)-1):
                out_header.append(header[idx[i]:idx[i+1]].strip())
            return out_header

def _convert_tbout_to_tab_delimiter(src,comments='#',raw_length=18):
    """ The number of elements in each line is constant equal to raw_length.
        No space was found in elements except the last one.
    """
    for line in src:
        line = line.strip()
        if line[0] == comments:
            continue
        line_array = line.split()
        last_element = " ".join(line_array[raw_length-1:])
        line_array2 = line_array[:raw_length-1]
        line_array2.append(last_element)
        yield line_array2

def _pick_best_one(df):
    """ While multiple alignment can be found for one certain sequence,
        a representative record should be chosen as the main feature of the source sequence.
        Criterion is based on E-value.
    """
    output = []
    for k,gp in df.groupby("query_name"):
        # Get min Evalue rows,
        # which could be more than one where two row both have minimal E-value and maximum score).
        gp = gp[gp.E_value == gp.E_value.min()]
        gp = gp[gp.score == gp.score.max()]
        if gp.shape[0] > 1:
            gp = gp.iloc[[1],:]
        output.append(gp)
    return pd.concat(output)

def read_Tbout_raw(source):
    """ Parser for tbout file without filtering step.
    """
    # for cmmscan
    #header =["target_name", "target_accession", "query_name", "query_accession", "mdl", "mdl_from", "mdl_to", "seq_from", "seq_to", "strand", "trunc", "pass", "gc", "bias", "score", "E_value", "inc", "description_of_target"]
    header = _tbout_get_header(open(source))
    df = pd.DataFrame.from_records(_convert_tbout_to_tab_delimiter(open(source), raw_length=len(header)))
    df.columns=header
    return df

def read_Tbout(source,threshold=0.0001,only_one=False):
    """ Parser for tbout file of cmscan program.
        The output file will be read as DataFrame object of pandas,
        following by filter out untrustable alignment accoding to the threshold on E-value.
        When only_one has set to True, Only one record with the smallest E-value and the
        largest score is retained per query.
    """
    df = read_Tbout_raw(source)
    df = df.astype({'E-value':'float32'})
    # Filter out untrustable alingments with a relative strict threshold.
    if threshold != 0:
        df = df[df['E-value']<threshold]
    if only_one:
        df = _pick_best_one(df)
    return df

def read_Rfam_family(source="/home/yduan/database/Rfam/Rfam_14.1/family.txt"):
    df = pd.read_csv(source,sep='\t',encoding = "ISO-8859-1",header=None)
    names = list(df.columns)
    names[0] = "accession"
    names[1] = "short_name"
    names[3] = "whole_name"
    names[9] = "description"
    names[18] = "type"
    df.columns = names
    return df

def read_spe_exp(source,drop_names = False):
    df = pd.read_csv(source,sep='\t',header=None,names=['time_type','tx_count','gene_count','tx_names'])
    # Both versions of split work.
    df['time'], df['type'] = df['time_type'].str.split(',').str
    #df = pd.concat((df,df['time_type'].str.split(',',expand=True).rename(columns={0:'time',1:'type'})),1)

    df = df.drop('time_type',1)
    # Reorder columns.
    df = df[['time', 'type', 'tx_count', 'gene_count', 'tx_names']]
    # drop tx_names
    if drop_names:
        df = df.drop('tx_names',1)
    return df

def _spe_exp_parse_name(name):
    """
    hq_lq_lnc_spe_f99.txt --> (hq_lq_lnc, f99)
    hq_lnc1_spe_1.txt --> (hq_lnc1, 1)
    """
    pt = re.compile(r'^(\S+)_spe_([^.]+)\.txt$')
    name = os.path.basename(name)
    return pt.match(name).groups()

def _read_spe_exp(source):
    count = read_spe_exp(source, drop_names = True)
    count['merge'], count['exp_thre'] = _spe_exp_parse_name(source)
    return count

def read_spe_exps(sources):
    """ Read specific expression files into one dataframe.
        And add two columns that are merge(hq_lq_lnc/hq_lnc1/hq_lnc2) and exp_thre(1/5/10...).
        Names are dropped by default.
    """
    data = []
    for s in sources:
        data.append(_read_spe_exp(s))
    return pd.concat(data)

def _get_stru(name):
    """ Get type of event from event_id.
    """
    id_desc = name.split(";")
    mytype = id_desc[1].split(":")[0]
    return mytype

def read_ioe_file(source):
    """Read ioe file and parse type of event."""

    df = pd.read_csv(source, sep='\t')
    df['event_type'] = df['event_id'].apply(_get_stru)
    return df
    



#!/usr/bin/env python3

import sys
from dypylib.bio.seq.base import Gff, GtfDict

'''
Count transcript and gene of input gffs
input: gffs
output:
gffname  gene_count tx_count mono_exon_tx two_exon_tx more_than_two_exon_tx tx_per_gene ratio_of_mono_exon_tx ratio_of_two_exon_tx ratio_of_more_than_two_exon_tx
'''

def count_tx_and_gene(gff):
    """ Return: count of gene; count of transripts; tx per gene.
    """
    #tx = set()
    #gene = set()
    #for rec in gff:
    #    if rec.type == "exon":
    #        tx.add(rec.tx_id)
    #        gene.add(rec.gene_id)
    #for chrdict in gff.values():
    #    print(k)
    #return len(gene), len(tx), len(tx)/len(gene)
    tx_1 = 0 # tx is constructed of 1 exon
    tx_2 = 0 # tx is constructed of 2 exons
    tx_n = 0 # tx is constructed of more than 3 exons
    tx_dict = gff.get_TxDict()
    for tx in tx_dict.values():
        if tx.get_count() == 1:
            tx_1 += 1
        elif tx.get_count() == 2:
            tx_2 += 1
        else:
            tx_n += 1
    tx_count = len(tx_dict)
    gene_count = len(gff.get_GeneDict())
    # empty gff??
    if tx_count == 0 or gene_count == 0:
        return 0,0,0,0,0,0,0,0,0
    return gene_count, tx_count, tx_count/gene_count, tx_1, tx_1/tx_count, tx_2, tx_2/tx_count, tx_n, tx_n/tx_count

def count(gffs,outfile,fm,sep = "\t",outnames=[],header=False):
    if header:
        outfile.write(sep.join(["gene_count", "tx_count", "tx_per_gene", "tx_1", "tx_1_fraction", "tx_2", "tx_2_fraction", "tx_n", "tx_n_fraction"])+"\n")
    for i in range(len(gffs)):
        gff=gffs[i]
        if outnames:
            name = outnames[i]
        else:
            name = ".".join(gff.split("/")[-1].split(".")[:-1])
        outfile.write(name+sep+sep.join(map(str,count_tx_and_gene(GtfDict(gff,fm=fm))))+"\n")

def main(argv):
    import argparse

    parser = argparse.ArgumentParser(description="Count tx and gens in gff")
    parser.add_argument('infiles',nargs='+',help="input gffs")
    parser.add_argument('-o','--outfile',nargs='?',help="output file",default=sys.stdout,type=argparse.FileType('w'))
    parser.add_argument('-n','--names',nargs='?',help="gff names. Seperated by comma.")
    parser.add_argument('-t','--gff-type',nargs='?',default='normal',choices=['normal','gc'])
    parser.add_argument('--header',action='store_true',help="Set this option to enable header output.")
    parser.add_argument('-s','--sep',nargs='?',default="\t")
    args = parser.parse_args(argv[1:])

    if args.names:
        names = args.names.split(",")
    else:
        names = []
    count(args.infiles,args.outfile,args.gff_type,args.sep,names,args.header)

if __name__ == '__main__':

    import sys
    main(sys.argv)



#!/usr/bin/env python3

import sys
from dypylib.bio.seq.base import Gff

'''
input gtf should only contain exon feature.
output length, count for exon or transcripts.
'''

def make_feature_dict(mygff):
    data={}
    for rec in mygff:
        data.setdefault(rec.tx_id,[]).append(rec)
    return data

def stat_db(mydb):
    #be fit for transcripts survey
    #data:(transcript name, exon counts, transcript length)
    #exon_length:length of each exon

    data=[]
    exon_length = []
    for key,value in mydb.items():
        count=len(value)
        exons_l=list(map(len,value))#length of each exon
        exon_length.extend(exons_l)
        length=sum(exons_l)
        data.append((key,count,length))
    return data,exon_length

def write_db_stat(stat_of_db,outfile,write_header=True,sep="\t"):
    header=sep.join(("Tx","count","length"))
    if write_header:
        outfile.write(header+"\n")
    for stat in stat_of_db:
        outfile.write(sep.join(map(str,stat))+"\n")

def write_exons_length(data,outfile):
    outfile.write("\n".join(map(str,data))+"\n")

def main(argv):
    import argparse, re
    parser = argparse.ArgumentParser(description="Get description for gtf file.")
    parser.add_argument('infile',nargs='?',help="input gtf file",type=argparse.FileType('r'))
    parser.add_argument('-o','--outprefix',nargs='?',help="output \
            prefix. Default: infile name that trimmed with .g[tf]f")
    parser.add_argument('-t','--gff-type',nargs='?',default='normal',choices=['normal','gc'])
    args = parser.parse_args(argv[1:])

    mygff=Gff(args.infile,fm=args.gff_type,check_sort=False)
    txdb = make_feature_dict(mygff)
    stat_of_tx, length_of_exons = stat_db(txdb)
    if args.outprefix:
        outprefix = args.outprefix
    else:
        outprefix = re.sub(r'\.g[tf]f$','',args.infile.name,flags=re.IGNORECASE)
    out_db_stat=open(outprefix+".tx_stat.txt",'w')
    out_exon_length=open(outprefix+".exon_len.txt",'w')
    write_db_stat(stat_of_tx,out_db_stat)
    write_exons_length(length_of_exons,out_exon_length)

if __name__ == '__main__':
    import sys
    main(sys.argv) 
    #mygff=Gff(sys.argv[1],fm='gc')
    #txdb = make_feature_dict(mygff)
    #stat_of_tx, length_of_exons = stat_db(txdb)
    #outprefix=sys.argv[2]
    #out_db_stat=open(outprefix+".tx_stat.txt",'w')
    #out_exon_length=open(outprefix+".exon_len.txt",'w')
    #write_db_stat(stat_of_tx,out_db_stat)
    #write_exons_length(length_of_exons,out_exon_length)



#!/usr/bin/env python

import xml.etree.ElementTree as ET
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as ss
from scipy.optimize import curve_fit
from matplotlib.backends.backend_pdf import PdfPages
import matplotlib.patches as mpatches
from matplotlib import rc

class Dist(object):

    '''
    object of distribution data which is convenient for ploting histogram graph.
    attrs {width:xxx,...}
    bins [1,20,44,...]
    '''
    def __init__(self,bins,attrs={}):
        assert (isinstance(bins,list) or isinstance(bins,tuple))
        self.__bins = tuple(map(int,bins))
        self.load_attr(attrs)

    def load_attr(self,attrs):
        if isinstance(attrs,dict):
            self.attrs = attrs
        #assume paired list or tuple.
        else:
            self.attrs = {}
            for key,value in attrs:
                self.add_attr(key,value)

    def add_attr(self,key,value):
        if key in self.attrs.keys():
            print("Warning : %s appears more than once.")
        self.attrs[key] = value

    def get_bins(self):

        return self.__bins

    def get(self,key):

        return self.attrs.get(key)

class Cate(Dist):

    '''
    Category data
    '''

    def __init__(self,counts,labels,attrs={}):
        self.__counts = counts
        self.__labels = labels
        self.load_attr(attrs)

    def get_counts(self):
        return self.__counts

    def get_labels(self):
        return self.__labels

def suffix_end(name,end=".pdf"):

    if name.endswith(end):
        return name
    else:
        return name+end

#Get size of a two dimensional table which have enough room to place elements of a certain length.
#eg1 10 -> (4,3)
#eg2 9 -> (3,3)
#eg3 15 -> (4,4)
def two_dimse(length):

    sq = int(length ** 0.5)
    if sq * sq == length:
        return sq, sq
    elif (sq+1) * sq >= length:
        return sq+1, sq
    elif (sq+1) * sq < length:
        return sq+1, sq+1
    else:
        raise ValueError("Programing Error")

#Iter leaves node(with no child element) of an element object.
def iter_leaves(element):

    for i in element:
        #with no child
        if len(i) == 0:
            yield i

#Get all leaves node(with no child element) of an element object.
def get_leaves_v(element):

    #[(key,value),(key,value),...]
    data = []
    for child in iter_leaves(element):
        data.append(get_raw_stat(child))
    return data

#Remove name space prefix of a given name.
def clean_name(name):

    simbol = "}"
    name = name.split(simbol)[1]
    return name

#Get element name and it's text.
def get_raw_stat(element):

    return clean_name(element.tag), element.text

#Convert an element object to a Dist object.
def get_Dist(element):

    attrs  = get_leaves_v(element)
    elmt_bin = element.find("ns:BinCounts",ns)
    bins = [i.text for i in elmt_bin]
    myDist = Dist(bins,attrs)
    return myDist

#Convert an element object to a Category data.
def get_Cate(element):

    attrs  = get_leaves_v(element)
    counts = [int(i.text) for i in element.find("ns:BinCounts",ns)]
    labels = [i.text for i in element.find("ns:BinLabels",ns)]
    mycate = Cate(counts,labels,attrs)
    return mycate

#Compute GC contant from a root element.
def get_GC(root):

    fractions = root.findall("Stats:TotalBaseFractionPerChannel",ns)
    frac_label = map(lambda x:x.get("Channel"),fractions)
    frac_value = map(lambda x:float(get_raw_stat(list(x)[0])[1]),fractions)
    ifrac_G = frac_label.index("G")
    ifrac_C = frac_label.index("C")
    GC_contant = frac_value[ifrac_G]+frac_value[ifrac_C]

    return GC_contant

#write paired data to a tabular in LaTex format
def write_tab_tex(data):

    #introduction
    intro = r"\begin{tabular}{|c|c|}"
    intro_end = r"\end{tabular}"
    hl = r"\hline "
    rhl = r"\\\hline "
    sep = r"&"
    text = r""
    text += (intro+hl)
    for key, value in data:
        text += str(key)+sep+str(value)+rhl
    text += intro_end

    return text

#plot descriptions in a tabular format.
def plot_desc(pp,des):

    rc('text',usetex=True)
    fig = plt.figure(figsize=(10,10))
    mytext = write_tab_tex(des)
    ax = fig.add_axes([0,0,1,1])
    fig.text(0.15,0.55,mytext,fontsize=20,transform=ax.transAxes)
    fig.suptitle("Basic information",fontsize=30)
    pp.savefig(fig)
    rc('text',usetex=False)

#Plot a histogram with a Dist object at ax level.
def plot_dist(mydist,ax,color="r",plot_subtitle=True,short_subtitle=True):

    mybins=mydist.get_bins()
    width=float(mydist.get("BinWidth"))
    minbin=float(mydist.get("MinBinValue"))
    maxbin=float(mydist.get("MaxBinValue"))
    sampleMean=float(mydist.get("SampleMean"))
    sampleStd=float(mydist.get("SampleStd"))
    numbins = int(mydist.get("NumBins"))

    #get mid value of bin x value.
    x_bins0 = np.arange(minbin,minbin+(numbins+0.5)*width,width)
    x_bins = (x_bins0[1:]+x_bins0[:-1])/2
    midbin = mybins[len(mybins)/2]
    plt.hist(x_bins,len(mybins),weights=mybins,color=color)
    if plot_subtitle:
        subt = mydist.get("MetricDescription")
        if short_subtitle:
            subt = subt.split(":")[1]
        ax.set_title(subt)
    ax.text(0.7,0.7,"mean:%.2f\nstd:%.2f"%(sampleMean,sampleStd),transform=ax.transAxes)
    return ax

##ax level.
#Plot a pie with a Cate object at ax level.
def plot_pie(mycate,ax,colors=[],plot_subtitle=False):

    counts = mycate.get_counts()
    labels = mycate.get_labels()
    total = sum(counts)
    plt.pie(counts,labels=labels,autopct=lambda(p):'{:.2f}% ({:.0f})'.format(p,round(p*total/100)))
    if plot_subtitle:
        ax.set_title(mycate.get("MetricDescription"))
    return ax

#Plot a histogram figure with one Dist object.
def plot_sgl_dist(mydist,fig,header):

    fig.suptitle(header,fontsize=20)
    ax = fig.add_subplot(1,1,1)
    plot_dist(mydist,ax,plot_subtitle=False)

#Plot four histogram in one figure with four Dist object.
def plot_qua_dists(mydists,fig,header):

    fig.suptitle(header,fontsize=20)
    handles = []
    labels = []
    for i in range(4):
        ax = fig.add_subplot(2,2,i+1)
        ax = plot_dist(mydists[i],ax,color=colors[i])

##figure level.
#Plot a pie figure with one Cate object.
def plot_cate_dist(mycate,fig,header=""):

    fig.suptitle(header,fontsize=20)
    ax = fig.add_subplot(1,1,1)
    plot_pie(mycate,ax,plot_subtitle=False)

#plot multi-pie plot in one figure with Catas.
def plot_cate_dist_one(mycates,fig,header=""):

    fig.suptitle(header,fontsize=20)
    size = len(mycates)
    x, y = two_dimse(size)
    for i in range(size):
        ax = fig.add_subplot(x,y,i+1)
        plot_pie(mycates[i],ax,plot_subtitle=True)
    
##data level.
#Put all category plots into one figure, set onePlot=True
def plot_cate_figs(pp,data,onePlot=False,suptitle="Types"):

    if onePlot:
        mycates = []
        fig = plt.figure(figsize=(10,10))
        
    for statn,title in data:
        myCate = get_Cate(root.find("Stats:"+statn,ns))
        if onePlot:
            mycates.append(myCate)
        else:
            fig = plt.figure(figsize=(10,10))
            plot_cate_dist(myCate,fig,title)
            pp.savefig(fig)
    if onePlot:
        plot_cate_dist_one(mycates,fig,suptitle)
        pp.savefig(fig)

#plot single figs(one subplot only).
def plot_sgl_figs(pp,data):

    for statn,title in data:
        fig = plt.figure(figsize=(10,10))
        myDist = get_Dist(root.find("Stats:"+statn,ns))
        plot_sgl_dist(myDist,fig,title)
        pp.savefig(fig)

#plot qua figs in which each contains 4 subplots for different base. Data [(stats name,title),()]
def plot_qua_figs(pp,data):

    for statn,title in data:
        fig = plt.figure(figsize=(10,10))
        myDists = map(get_Dist,root.findall("Stats:"+statn,ns))
        plot_qua_dists(myDists,fig,title)
        pp.savefig(fig)

#input parameters
inFile = "/home/yduan/temp/1_G02/m54061_170910_110737.sts.xml"
outfile = "sts_test.pdf"

def plot_sts(infile,outfile):

    #Set Global parameters 
    global ns 
    ns = {'Stats':"http://pacificbiosciences.com/PacBioPipelineStats.xsd",'xsi':"http://www.w3.org/2001/XMLSchema-instance",'ns':"http://pacificbiosciences.com/PacBioBaseDataModel.xsd"}
    plt.style.use('ggplot')
    global colors
    colors =['m','c','y','g','b','r','b']
    #Parse xml-formated infile
    tree = ET.parse(infile)
    global root 
    root = tree.getroot()

    #Get plot data
    #get informations
    text_desc = get_leaves_v(root)
    GCcontant = get_GC(root)
    text_desc.append(("GCcontant",str(GCcontant)))

    sig_data = [("PausinessDist","Pausiness"),("PulseRateDist","Pulse Rate"),("PulseWidthDist","Pulse Width"),("BaseRateDist","BaseRate"),("BaseWidthDist","Base Width"),("BaseIpdDist","Base Ipd"),("LocalBaseRateDist","Local Base Rate"),("NumUnfilteredBasecallsDist","Unfiltered Base Calls"),("ReadLenDist","Read Length"),("InsertReadLenDist","Insert Read Length"),("MedianInsertDist","Median of Insert"),("HqBaseFractionDist","HQ Base Fraction")]
    qua_data = [("BaselineLevelDist","Base Line Level"),("BaselineStdDist","Base Line Std"),("SnrDist","Signal to Noise ratio"),("HqRegionSnrDist","Signal to Noise ratio(HQ region)"),("HqBasPkMidDist","HQ region PkMid"),("DmeAngleEstDist","DME Angle Estimation")]
    cate_data = [("ProdDist","Productivity"),("LoadingDist","Loading"),("ReadTypeDist","ReadType")]

    #ploting
    pp = PdfPages(outfile)
    plot_desc(pp,text_desc)
    plot_qua_figs(pp,qua_data)
    plot_sgl_figs(pp,sig_data)
    plot_cate_figs(pp,cate_data,onePlot=True)
    pp.close()

def main(argv):

    import argparse

    parser = argparse.ArgumentParser(description="Visualization tool for PacBio sts.xml file.")
    parser.add_argument('infile',nargs='?',default=sys.stdin,help="*sts.xml",type=argparse.FileType('r'))
    parser.add_argument('-o','--outfile',nargs='?',help="output file",default="sts_test.pdf")
    args = parser.parse_args(argv[1:])

    plot_sts(args.infile,suffix_end(args.outfile))

if __name__ == '__main__':

    import sys
    main(sys.argv)




#!/usr/bin/env python

'''
License: GNU General Public License v3.0 (http://www.gnu.org/licenses/gpl-3.0.html)
Author: Mr. You Duan
Email: yduan@outlook.com

classes: Fasta Seq Gff Gff_rec
functions: write_fasta 
'''

import sys, re, string, itertools, copy
import gffutils, sqlite3
from collections import OrderedDict as Ordic
try:
    from pygtftk.gtf_interface import GTF as tkGTF
    from pygtftk.Line import Feature as tkFeature
    import pysnooper
except:
    pass

try:
    from case_insensitive_dict import CaseInsensitiveDict
except:
    pass

vs=sys.version

class Seq(str):

    alphabet_nucleotide = set(["A","T","G","C","a","t","g","c"])
    alphabet_nucleotide_ext = set(["R","Y","M"])
    alphabet_nucleotide_ext.update(alphabet_nucleotide)

    def __new__(self,string,*args,**kwargs):

        return str.__new__(self,string)

    def __init__(self,string,name=None,*args,**kwargs):

        self.__name = name

    @property
    def name(self):
        return self.__name

    @name.setter
    def name(self,value):
        self.__name = value

    def __getitem__(self,key):
        return Seq(super(Seq,self).__getitem__(key),self.name)
    # python version 2.x
    def __getslice__(self,*args):
        return Seq(super(Seq,self).__getslice__(*args),self.name)

    def getRC(self):

        '''
        get reverse compliment sequence
        '''

        intab = 'AaTtGgCc'
        outtab = 'TtAaCcGg'
        if vs < '3.0':
            transtab = string.maketrans(intab,outtab)
        else:
            transtab = str.maketrans(intab,outtab)
        return Seq(self[::-1].translate(transtab),name=self.name)

    def isnucleotide(self,strict=True):
        if strict:
            _albet = Seq.alphabet_nucleotide
        else:
            _albet = Seq.alphabet_nucleotide_ext
        for _ in self:
            if _ not in _albet:
                return False
        return True

    def get_freq(self,alpha="g"):
        """ Get the frequency of the alpha.
        """
        ts = self.lower()
        alpha = alpha.lower()
        freq = ts.count(alpha)/len(ts)
        return freq

    def get_gc(self):
        gc = self.get_freq("g") + self.get_freq("c")
        return gc

    def to_fasta(self,outfile,lth=80):
        if not self.__name:
            write_fasta_s(outfile,self,"seqNull",lth)
        else:
            write_fasta_o(outfile,self,lth)

    def write2fasta(self,outfile,lth=80):
        sys.stderr.write("write2fasta will retired, use to_fasta replace\n")
        self.to_fasta(outfile,lth)

class Gff(object):
    """
        Gff iterator.
    """

    gff3_partern = re.compile(r"\s*([^\s=]+)[\s=]+(.*)")

    def __init__(self,infile,mode=1,sep="\t",fm="normal",check_sort=False):
        self.sep = sep
        self.fm = fm
        self.filename, self.infile = open_file(infile)
        # Use to check duplicated gene/tx.
        self.check_sort = check_sort
        if self.check_sort:
            self.itered_tx = set()
            self.curr_tx = ''
        
        if mode == 2:
            self.__data = []
            self.isiter = False
            self.loadfile()
        if mode == 1:
            self.isiter = True#this object can be looked as an iterator

    def __iter__(self):
        return self.iterator()

    def check_exist(self, name, name_set):
        if name in name_set:
            return True
        name_set.add(name)
        return False

    def if_gff_sorted(self, rec):
        # Tx_id information is not available.
        if self.fm == 'none':
            return 
        if rec.tx_id == self.curr_tx:
            return
        if self.check_exist(rec.tx_id, self.itered_tx):
            # gff is not sorted
            error_info = ('Tx: "{}" is not continuous '
                'in file\n').format(rec.tx_id)
            sort_help=('Input gff/gtf is not sorted\nYou can sort gtf'
                'with the following order:\ncat input.gtf|cgat'
                'gtf2gtf --method=sort --sort-order "'
                'gene+transcript" > sorted.gtf\n')
            error_info += sort_help
            raise IndexError(error_info)
        else:
            # Meet a new tx.
            # Update current transcript.
            self.curr_tx = rec.tx_id

    def iterator(self):
        if not self.isiter:
            #should change to mode 1
            raise TypeError("Fasta mode should change to 1")
        return self.__iterGff()
    
    def next(self):
        return self.iterator().next()

    def __iterGff(self):
        for line in self.infile:
            line=line.strip()
            if len(line)==0 or line[0]=="#":
                continue
            #yield Gff_rec(line.split(self.sep))
            rec = Gff_rec(line.split(self.sep),self.fm)
            if self.check_sort:
                self.if_gff_sorted(rec)
            yield rec

    def loadfile(self):
        #wait for compete
        raise TypeError("object Gff has not stock mode")
    
    @classmethod
    def Parse_attr(self,attr,attr_partern=gff3_partern):
        '''Parses Gff attribution string and results it as a dictionary
        '''

        #attr_dic = {}
        attr_dic = Ordic()
        #tmplst = map(lambda x:attr_partern.match(x),attr.strip().split(";")[:])
        tmplst = map(lambda x:attr_partern.match(x),attr.strip().split(";"))
        if vs > '3.0':
            tmplst = list(tmplst)
        if not tmplst[-1]:
            tmplst.pop()#in some case,";" exists as the last charactor, which will cause a None item in tmplst[-1]
        for each in tmplst:
            if not each:
                print("gff type error")
                sys.exit()
            val = each.group(2)
            if val.startswith('"') and val.endswith('"'):
                val = val[1:-1]
            attr_dic.setdefault(each.group(1),val)
        return attr_dic

class GffRec(Seq):

    def __init__(self, lst, fm):
        if fm == "normal":
            return Gff_rec(self, lst)

class Gff_rec(Seq):

    '''gff records
    '''
    def __init__(self,lst,fm,chr=0,sour=1,type=2,start=3,end=4):
    
        self.chr = lst[chr]
        self.Chr = self.chr
        self.source = lst[sour]    
        self.type = lst[type]
        self.start = int(lst[start])
        self.end = int(lst[end])
        self.score = lst[5]
        self.strand = lst[6]
        self.phase = lst[7]
        self.attr = Gff.Parse_attr(lst[8])
        self.lst = lst
        self.seq = ''
        self.format_attr(fm)

    #def __getattr__(self,name):
    #    raise KeyError("%s don\'t have an attribute named %s"% ("self.attr",name))
    
    def __str__(self,sep="\t",version=2):
        data = self.get_lst()
        data[-1] = self.package_attr(version)
        return sep.join(map(str,data))

    def __len__(self):
        return abs(int(self.end)-int(self.start))+1

    def as_str(self,sep="\t",version=2):
        return self.__str__(sep,version)

    def format_none(self):
        self.gene_id = ""
        self.tx_id = ""

    def format_normal(self):
        # gene records don't have gene_id attribution
        self.gene_id = self.attr.get("gene_id")
        # gene records don't have transcript_id attribution
        self.tx_id = self.attr.get("transcript_id")

    # Grass carp gff format
    def format_gc(self):
        # Warning: rstrip should be replaced.
        self.gene_id = self.attr["ID"].rstrip(".EXON")
        self.tx_id = self.gene_id

    def format_attr(self,fm):
        if fm == "normal":
            self.format_normal()
        elif fm == "gc":
            self.format_gc()
        elif fm == "none":
            self.format_none()
        else:
            raise KeyError("Wrong file format code: %s" % fm)

    # Refers to Ensembl GTF
    def pack_attr_gtf(self):
        data = []
        for key, val in self.attr.items():
            data.append(key+" \""+val+"\"")
        return "; ".join(data)+";"

    def edit_gene_id(self,val):
        self.gene_id = val
        # Raise unknown results when format is not normal.
        self.attr["gene_id"] = val

    def edit_tx_id(self,val):
        self.tx_id = val
        # Raise unknown results when format is not normal.
        self.attr["transcript_id"] = val

    # Refers to Ensembl GFF
    def pack_attr_gff(self):
        data = []
        for key, val in self.attr.items():
            data.append(key+"="+val)
        return "; ".join(data)

    def package_attr(self,gff_version=2):
        if gff_version == 2:
            return self.pack_attr_gtf()
        elif gff_version == 3:
            return self.pack_attr_gff()
        else:
            raise KeyError("Version can be only 2 or 3.\n")

    def get_lst(self):
        return [self.chr,self.source,self.type,self.start,self.end,\
            self.score,self.strand,self.phase,self.attr]

# Genome element
class GENT(object):

    def __init__(self,start,end,strand=".",attr={}):
        self.start = int(start)
        self.end = int(end)
        # if start end has been adjusted, in which end >= start.
        self.strand = strand
        self._guess_strand()
        self._adj_pos()
        self.attr=attr

    def __len__(self):
        return abs(self.end - self.start) + 1

    def _guess_strand(self):
        if self.start > self.end:
            if self.strand == "+":
                raise ValueError("Stand is not suit for coordinates.\n")
            self.strand = "-"

    def _adj_pos(self):
        if self.start > self.end:
            self.start, self.end = self.end, self.start

class EXON(GENT):

    def __init__(self,*args,**kwargs):
        """ Bugs:
            When the attibution has been altered, the correponding value in self.rec keeps original.
            Need a update_rec function to solve this problem.
        """
        if isinstance(args[0],Gff_rec):
            o = args[0]
            self.init(o.start,o.end,o.strand,o.Chr,o.gene_id,o.tx_id)
            self.rec = o
        else:
            self.init(*args,**kwargs)
            self.rec = None

    def init(self,start,end,strand,Chr,gene_id,tx_id):
        self.Chr = Chr
        self.gene_id = gene_id
        self.tx_id = tx_id
        super(EXON,self).__init__(start,end,strand)

    def edit_gene_id(self,val):
        """ Alter gene_id
        """
        self.gene_id = val
        self.rec.edit_gene_id(val)

    def edit_tx_id(self,val):
        """ Alter tx_id
        """
        self.tx_id = val
        self.rec.edit_tx_id(val)

    def update_rec(self):
        """ Under development...
        """
        pass

    def as_str(self):
        return self.rec.as_str()

    def as_gtf(self):
        return self.as_str()

class INTRON(GENT):

    def __init__(self,start,end,strand,Chr,gene_id = None,tx_id = None):
        self.Chr = Chr
        self.chrom = Chr
        self.gene_id = gene_id
        self.tx_id = tx_id
        super(INTRON,self).__init__(start,end,strand)

    def __hash__(self):
        return hash((self.start, self.end, self.strand, self.Chr, \
                self.gene_id, self.tx_id))

    def __eq__(self, other):
        try:
            return (self.Chr == other.Chr) and (self.strand == other.strand) and \
                   (self.start == other.start) and (self.end == other.end)
        except:
            return False

class SGENT(dict,GENT):
    """Segments genome element
    like: transcripts, genes.
    When parse gff record, basic information like start, end, strand will be load first.
    While comes across gtf file, basic infromation was acquired through add_exon methods.
    """
    def __new__(self,*args,**kwargs):
        return dict.__new__(self)

    def __init__(self,es=[],perm_op=True,static=False,start=float("inf"),end=-float("inf"),strand=".",rec=None):
        # es: elements
        # perm_op: permit elements overlap
        # static: when static is True, start, end, strand, length option should not be update by add_exon.
        if rec == None:
            self._init_from_GENTs(es,perm_op,static,start,end,strand)
        elif isinstance(rec,Gff_rec):
            self._init_from_rec(rec,perm_op)
        else:
            sys.stderr.write("rec type is wrong\n")
            sys.exit(1)

    def __len__(self):
        return self.get_length()

    def _init_from_GENTs(self,es,perm_op,static,start,end,strand):
        self.start = start
        self.end = end
        self.strand = strand
        if static:
            self._init_static()
        else:
            self.static = False
            self._length = 0
        self._count = 0
        #self.extend(es)
        # Whether to permit elements overlap
        self.perm_op = perm_op

        # Add single element
        if type(es) != list:
            es = [es]
        self.add_es(es)

    def _init_static(self):
        self._adj_pos(self)
        self._length = self.start - self.end
        self.static = True

    def _init_from_gffutils(self, db, key):
        """Init Object from a gffutils database"""
        pass

    def _init_from_rec(self,rec,perm_op):
        # Bugs!!!
        self.static = True
        self.start = rec.start
        self.end = rec.end
        self.strand = rec.strand
        # maybe only gc gff
        self.ID = rec.attr.get("ID")
        self.name = rec.attr.get("Name")
        self._count = 0
        self.perm_op = perm_op

    def add_es(self,es):
        for e in es:
            self.add_e(e)

    def _update_info(self,e):
        if self.static:
            sys.stderr.write("Wrong update calling.\n")
        self._length += len(e)
        self.start = min(self.start, e.start)
        self.end = max(self.end, e.end)

    def add_e(self,e):
        self.check_strand(e)
        self.check_overlap(e)
        self.add_data(e)
        self.add_len(e)
        if not self.static:
            self._update_info(e)

    def check_strand(self,e):
        if self.strand == ".":
            self.strand = e.strand
        if self.strand != e.strand:
            raise ValueError("strand of exon and transcript should be identical.\n")

    # Append
    def add_data(self,e, ID = None):
        if not ID:
            try:
                ID = e.ID
            except:
                ID = self.get_count()
        if ID in self.keys():
            raise KeyError("Reapeat ID, will cause cover issue.\n")
        self[ID] = e

    def add_len(self,e):
        # non-overlap
        self._count += 1

    def check_overlap(self,e2):
        if self.perm_op:
            return
        if check_overlap_e(self,e2):
            sys.stderr.write("%s, %s, %s\n"%(e2.Chr,e2.gene_id,e2.tx_id))
            sys.stderr.write("start1, end1, start2, end2 = %d,%d,%d,%d\n"%(self.start,self.end,e2.start,e2.end))
            raise TypeError("Overlapped elements are not permited.\n")

    def any_overlap(self):
        tmp = sorted(self.values(),key = lambda x:x.start)
        for i in range(self.get_count()-1):
            if check_overlap_e(tmp[i],tmp[i+1]):
                return True
        return False

    # Get transcript start site.
    def get_tss(self):
        return self.get_5p_end()

    def get_tes(self):
        return self.get_3p_end()

    def get_5p_end(self):
        return self._get_tss()

    def get_3p_end(self):
        return self._get_tes()

    def _get_tss(self):
        st = self.start
        ed = self.end
        if st > ed:
            st, ed = ed, st
        if self.strand == "-":
            return ed
        else:
            return st

    # Get transcript end site.
    def _get_tes(self):
        st = self.start
        ed = self.end
        if st > ed:
            st, ed = ed, st
        if self.strand == "-":
            return st
        else:
            return ed

    def get_count(self):
        return self._count

    def get_length(self):
        return self._length

    def get_sorted_values(self,key = lambda x: x.start):
        return sorted(self.values(),key = key)

# transcript
class TxDict(SGENT):

    def __init__(self,*args,**kwargs):
        kwargs["perm_op"] = False
        super(TxDict,self).__init__(*args,**kwargs)

    def add_exon(self,exon):
        self.add_e(exon)

    def add_e(self,exon):
        assert isinstance(exon,EXON)
        self.ID = exon.tx_id
        self.Chr = exon.Chr
        self.gene_id = exon.gene_id
        super(TxDict,self).add_e(exon)

    def edit_gene_id(self,val):
        """ Alter gene_id
        """
        self.gene_id = val
        for exon in self.values():
            exon.edit_gene_id(val)

    def edit_tx_id(self,val):
        """ Alter tx_id
        """
        self.ID = val
        for exon in self.values():
            exon.edit_tx_id(val)

    def as_gtf(self):
        """ Get string in GTF format.
        """
        return "\n".join(map(lambda x:x.as_gtf(),self.values()))

    def get_introns(self):
        """ Return a list of introns of this transcript.
        """
        res = []
        exons = sorted(list(self.values()),key=lambda x:x.start)
        for i in range(len(exons)-1):
            e_0 = exons[i]
            e_1 = exons[i+1]
            _start = e_0.end + 1
            _end = e_1.start - 1
            if _start > _end:
                raise ValueError("The end of the intron must be bigger than the start")
            res.append(INTRON(_start,_end,e_0.strand,e_0.Chr,e_0.gene_id,e_0.tx_id))
        return res

class GeneDict(SGENT):

    def add_exon(self,exon):
        if exon.tx_id in self.keys():
            # transcript add exon
            self[exon.tx_id].add_exon(exon)
            super(GeneDict,self)._update_info(exon)
        else:
            self.add_e(TxDict([exon]))

    def add_tx(self,tx):
        self.add_e(tx)

    def add_e(self,tx):
        assert isinstance(tx,TxDict)
        self.ID = tx.gene_id
        self.Chr = tx.Chr
        super(GeneDict,self).add_e(tx)

    def edit_gene_id(self,val):
        """ Alter gene_id
        """
        self.ID = val
        for tx in self.values():
            tx.edit_gene_id(val)

    def merge(self,e,new_id=None):
        """ merge self and SGENT e.
        """
        out = copy.deepcopy(self)
        out.start = min(self.start,e.start)
        out.end = max(self.end,e.end)
        out.update(e)
        if new_id:
            out.edit_gene_id(new_id)
        return out

    def as_gtf(self):
        """ For GTF
        """
        return "\n".join(map(lambda x:x.as_gtf(),self.values()))

    def get_represent_tx(self):
        """ Get representative transcript of one gene, usally the longest one.
            Useful when doing gene function annotation."""
        max_len = 0 
        for tx in self.values():
            if len(tx) > max_len:
                rep_tx = tx
                max_len = len(tx)
        return rep_tx


class ChrDict(SGENT):

    def __init__(self,es=[],perm_op=True):
        # es: elements
        # perm_op: permit elements overlap
        self.start = float("inf")
        self.end = - float("inf")
        self._length = 0
        self._count = 0
        #self.extend(es)
        # Whether to permit elements overlap
        self.perm_op = perm_op

        # Add single element
        if type(es) != list:
            es = [es]
        self.add_es(es)

    def add_exon(self,exon):
        if exon.gene_id in self.keys():
            self[exon.gene_id].add_exon(exon)
        else:
            self.add_e(GeneDict(TxDict([exon])))

    def add_tx(self,tx):
        if tx.gene_id in self.keys():
            self[tx.gene_id].add_tx(tx)
        else:
            self.add_e(GeneDict([tx]))

    def add_gene(self,gene):
        self.add_e(tx)

    def add_e(self,gene):
        assert isinstance(gene,GeneDict)
        self.ID = gene.Chr
        #super(ChrDict,self).add_e(gene)
        self.add_data(gene)
        self.add_len(gene)

    def sort(self):
        self.sort(key=lambda x:x.start)

    def __len__(self):
        return self.get_count()

class GenomeDict(SGENT):
    """to-do: Remove this object"""

    def add_exon(self,exon):
        if exon.Chr in self.keys():
            self[exon.Chr].add_exon(exon)
        else:
            self.add_e(ChrDict(GeneDict(TxDict([exon]))))

    def add_e(self,gene):
        assert isinstance(gene,ChrDict)
        super(GenomeDict,self).add_e(gene)

    def add_gene(self,gene):
        self.add_e(ChrDict(gene))

    def add_gene_rec(self,rec):
        gene = GeneDict(rec=rec)

    def check_strand(self,e):
        pass

    def __len__(self):
        return self.get_count()

class GTF(object):

    """
    A wraper for tkGTF

    """

    def __init__(self, input_obj, ID=None, *argv, **kwargv):
        """
        The constructor

        Since tkGTF can be constructed with both File, (tk)GTF and string object, the input_obj can be any of them, theoretically.
        Because tkGTF is slow, it is time comsuming to reconsctruct a GTF object, while a wrapper is more efficient.

        :param input_obj: tkGTF object.
        "init_children bool: Set to True to call self.__init_children.
        """
        self.wrapped_class = input_obj
        self.ID = ID

    def __getattr__(self,attr):
        orig_attr = self.wrapped_class.__getattribute__(attr)
        # why if callable???
        #if callable(orig_attr):
        #    return orig_attr
        if(self == self.wrapped_class):
            print("self == wrapped")
        return orig_attr

    def __getitem__(self,key):
        return self.get(key)

    def __str__(self):
        """
        Make the output of print looks like:

        - A GTF object containing 14 lines:
        - File connection: tmp/tttt_met_eb.gtf
        
        CI01000000	0,2,2	gene	252526	259879	1000.00	+	.	gene_id	"CIWT.10";
        CI01000000	0,2,2	transcript	252526	259876	1000.00	+	.	transcript_id "CIWT.10.1"; gene_id "CIWT.10";
        CI01000000	0,2,2	exon	252526	252935	1000.00	+	.	transcript_id "CIWT.10.1"; gene_id "CIWT.10";
        CI01000000	0,2,2	exon	253671	253903	1000.00	+	.	transcript_id "CIWT.10.1"; gene_id "CIWT.10";
        CI01000000	0,2,2	exon	256634	256857	1000.00	+	.	transcript_id "CIWT.10.1"; gene_id "CIWT.10";
        CI01000000	0,2,2	exon	257469	257660	1000.00	+	.	transcript_id "CIWT.10.1"; gene_id "CIWT.10";

        Otherthan:
        <bio.seq.base_md.GTFTx object at 0x7fd251e3e358>
        """
        return str(self.wrapped_class)

    @staticmethod
    def get_first_feature(input_obj):
        """
        Get the first tkFeature object of a tkGTF
        using iteration.

        :param input_obj: A tkFeature object
        """
        assert isinstance(input_obj, tkGTF)
        for f in input_obj:
            return f

    @staticmethod
    def get_key_feature_obj(input_obj, key):
        """
        Help to get a primary Feature object.
        For-loop of python is much more faster than get_key_feature of tkGTF.

        :param input_obj: A tkFeature object
        :param key: feature name, like transcript, gene and exon.
        """
        if isinstance(input_obj, tkGTF):
            for rec in input_obj:
                if rec.ft_type == key:
                    return rec
        elif isinstance(input_obj,tkFeature):
            return input_obj
        elif isinstance(input_obj, GTF):
            for rec in input_obj.wrapped_class:
                if rec.ft_type == key:
                    return rec
        else:
            print(type(input_obj))
            raise TypeError

    @staticmethod
    def _get_key_feature_obj(input_obj, key):
        """
        Help to get a primary Feature object.

        :param input_obj: A tkFeature object
        :param key: feature name, like transcript, gene and exon.
        """
        if isinstance(input_obj,tkFeature):
            return input_obj
        mygtf = input_obj.select_by_key('feature',key)
        return GTF.get_first_feature(mygtf)
    
class iGTF(GTF):
    """
    Subclass of GTF, the iterable version of GTF.
    """

    def __init__(self, input_obj, ID=None, init_children=True,as_dict=False,*argv,**kwags):
        super(iGTF,self).__init__(input_obj, ID=ID)
        if init_children:
            self._init_children()
            self._length = len(self._keys)

        # If as_dict, store hash relationship.
        # Else, call value by dynamic search.
        self.is_dict = False
        if as_dict:
            self._init_dict()

    def _init_children(self):
        """
        Crucial for iteration.
        Called when init_children is set to True in __init__
        """
        self._keys = self.get_chroms(nr=True)

    def _init_dict(self):
        """
        Init a static dictionary, which can accelerate the extracting operation.
        """
        self._dict = Ordic()
        for key, val in self.items():
            self._dict[key] = val
        self.is_dict = True

    def __len__(self):
        return self._length

    def __iter__(self):
        if self.is_dict:
            return self._dict.values()
        else:
            return iter(self._keys)

    def items(self):
        if self.is_dict:
            return self._dict.items()
        
        for key in self:
            yield key, self[key]

    def keys(self):
        if self.is_dict:
            return self._dict.keys()
        else:
            return self._keys

    def values(self):
        if self.is_dict:
            return self._dict.values()

        for key in self:
            yield self[key]

class fGTF(GTF):
    """
    Featured GTF.
    A fGTF object always has a representative feature line in GTF(ensembl).

    :Example:
    GTFGene
    GTFTx
    GTFExon
    GTFCds
    """
    _fdic = {'chr':'chrom','chrom':'chrom','seqname':'chrom','seqid':'chrom',
             'source':'src','src':'src',
             'type':'ft_type','feature':'ft_type','ft_type':'ft_type',
             'start':'start',
             'end':'end',
             'score':'score',
             'strand':'strand',
             'phase':'frame','frame':'frame'}

    def __init__(self, input_obj, primary_rec=None, ID=None, *argv, **kwargv):
        """
        :param input_obj: Typically GTF object, sometimes Feature object.
        :param primary_rec: The represent Feature of GTF object, the first line will be used when None is given.
        :param ID: The same with GTF __init__.
        """
        super(fGTF,self).__init__(input_obj,ID, *argv, **kwargv)

        # Set primary_rec to call some attribution in _fdic using "." method.
        # When the input_obj is tkGTF, choose the first record as p_r when
        # the primary_rec isn't provided in the parameters.
        if isinstance(input_obj,tkFeature):
            self.primary_rec = input_obj
        elif isinstance(input_obj,tkGTF):
            self.primary_rec = primary_rec if primary_rec else GTF.get_first_feature(input_obj)
        else:
            raise TypeError("Only tkGTF and tkFeature is supported.")

    def __getattr__(self,attr):
        if attr in fGTF._fdic:
            return getattr(self.primary_rec, fGTF._fdic[attr])
        return self.wrapped_class.__getattribute__(attr)

    def __len__(self):
        return self.end - self.start

    # Get transcript start site.
    def get_tss(self):
        st = self.start
        ed = self.end
        if st > ed:
            st, ed = ed, st
        if self.strand == "-":
            return ed
        else:
            return st

    # Get transcript end site.
    def get_tes(self):
        st = self.start
        ed = self.end
        if st > ed:
            st, ed = ed, st
        if self.strand == "-":
            return st
        else:
            return ed

class eGTF(fGTF):

    """
    Base element GTF.
    A eGTF object always has a representative feature line in GTF(ensembl).

    :Example:
    GTFExon
    GTFCds
    """
    #def __len__(self):
    #    return self.end - self.start
    pass

class GTFCds(eGTF):pass

class GTFExon(eGTF):pass

class GTFTx(fGTF):
    """
    Object bind closely to GTFGene

    :Relationship:

    :Parent class: GTFGene
    :Child class: EXON

    """
    def __init__(self, input_obj, ID=None):
        """
        GTF.get_key_feature_obj function is very slow due to select_by_key.
        Hence, only use get_first_feature instead
        """
        super(GTFTx,self).__init__(input_obj,
                                   primary_rec = GTF.get_key_feature_obj(input_obj, 'transcript'),
                                   #primary_rec = GTF.get_first_feature(input_obj),
                                   ID=ID)

    def __len__(self):
        _length = 0
        for e in self:
            _length += len(e)
        return _length

    def __iter__(self):
        return iter(self.get_exons())

    def get_exons_old(self):
        """
        Select_by_key is extrimly slow, avoid using!
        """
        return map(GTFExon,self.select_by_key('feature','exon'))

    def get_cds_old(self):
        return map(GTFCds,self.select_by_key('feature','cds'))

    def _iter_type(self, key):
        for rec in self.wrapped_class:
            if rec.ft_type == key:
                yield rec

    def get_exons(self):
        for rec in self._iter_type('exon'):
            yield GTFExon(rec)

    def get_cds(self):
        for rec in self._iter_type('cds'):
            yield GTFCds(rec)

    def get_introns(self):pass

class GTFGene(iGTF,fGTF):
    """
    Object bind closely to GTFChr

    :Relationship:

    :Parent class: GTFChr
    :Child class: GTFTx

    """
    def __init__(self, input_obj, ID=None):
        super(GTFGene,self).__init__(input_obj,
                                     primary_rec = GTF.get_key_feature_obj(input_obj, 'gene'),
                                     #primary_rec = GTF.get_first_feature(input_obj, 'gene'),
                                     ID=ID,
                                     as_dict= True)

    def _init_children(self):
        """
        Crucial for iteration.
        Called when init_children is set to True in __init__
        """
        self._keys = self.get_tx_ids(nr=False)

    def get(self,key):
        """
        Core function for bunch of methods, including
        __getitem__, iteration, items(), keys(), values()
        
        Divergent child class which represent certain biology segement
        can be created by simply rewriting this function.
        """
        return GTFTx(self.select_by_key("transcript_id",key),key)

class GTFChr(iGTF):
    """
    Object bind closely to GTFGenome

    :Relationship:

    :Parent class: GTFGenome
    :Child class: GTFGene

    """
    def __init__(self, input_obj, ID=None):
        #super(GTFChr,self).__init__(input_obj,ID=ID,as_dict=True)
        super(GTFChr,self).__init__(input_obj,ID=ID)

    def _init_children(self):
        """
        Crucial for iteration.
        Called when init_children is set to True in __init__
        """
        self._keys = self.get_attr_value_list(key="gene_id")

    def get(self,key):
        """
        Core function for bunch of methods, including
        __getitem__, iteration, items(), keys(), values()
        
        Divergent child class which represent certain biology segement
        can be created by simply rewriting this function.
        """
        return GTFGene(self.select_by_key("gene_id",key),key)

    def get_tx_feature(self):
        return list(map(GTFTx,self.select_by_key('feature','transcript')))

    def get_txs(self):
        ids = self.get_tx_ids(nr=True)
        return list(map(lambda x:GTFTx(self.select_by_key('transcript_id',x)), ids))

class GTFGenome(iGTF):
    """
    Toplevel bio feature handle for GTF
    The constructor.

    :Relationship:

    :Depending class: tkGTF

    """
    def __init__(self,gtf):
        """
        :param gtf: File, tkGTF, or any other object accepted by tkGTF.
        """
        super(GTFGenome,self).__init__(tkGTF(gtf))

    def get(self,key):
        """
        Core function for bunch of methods, including
        __getitem__, iteration, items(), keys(), values()
        
        Divergent child class which represent certain biology segement
        can be created by simply rewriting this function.
        """
        return GTFChr(self.select_by_key("chr",key),key)

    def get_chr_to_x(self,e_type,strand=False):
        """ Get a list of {chr:[transcripts]}
            :param: e_type, element type, can be one of [tx, gene].
            :param: strand, set strand to true to include strand in
                key. The key will be (chr, strand) rather than chr.
        """
        if e_type == "tx":
            _key = "transcript_id"
            myclass = GTFTx
        elif e_type == "gene":
            _key = "gene_id"
            myclass = GTFGene

        my_dict = self.extract_data("chrom,"+_key,
                                    as_dict_of_merged_list=True,
                                    nr=True,
                                    hide_undef=True)
        outdict = Ordic()
        for key, val in my_dict.items():
            for v in val:
                e = myclass(self.select_by_key(_key,v),v)
                #if strand == True:
                outkey = (key,e.strand) if strand else key
                _vals = outdict.setdefault(outkey, [])
                _vals.append(e)
            
        return outdict

    def get_chr_to_tx(self,if_strand=False):
        return self.get_chr_to_x("tx",if_strand)

    def get_chr_to_gene(self,if_strand=False):
        return self.get_chr_to_x("gene",if_strand)

    #def get_chr_to_gene1(self,strand=False):
    #    my_dict = self.extract_data("chrom,gene_id",
    #                                as_dict_of_merged_list=True,
    #                                nr=True,
    #                                hide_undef=True)
    #    outdict = Ordic()
    #    for key, val in my_dict.items():
    #        _vals = []
    #        for v in val:
    #            _vals.append(GTFTx(self.select_by_key('gene_id',v),v))
    #        outdict[key] = _vals
    #        
    #    return outdict

    #def get_chr_to_tx1(self,strand=False):
    #    """ Get a list of {chr:[transcripts]}
    #        :param: strand, set strand to true to include strand in
    #            key. The key will be (chr, strand) rather than chr.
    #    """
    #    my_dict = self.extract_data("chrom,transcript_id",
    #                                as_dict_of_merged_list=True,
    #                                nr=True,
    #                                hide_undef=True)
    #    outdict = Ordic()
    #    for key, val in my_dict.items():
    #        for v in val:
    #            tx = GTFTx(self.select_by_key('transcript_id',v))
    #            if strand==True:
    #                key = (key,tx.strand)
    #            _vals = outdict.setdefault(key, [])
    #            _vals.append(tx)
    #        
    #    return outdict

class GffDict(GenomeDict):
    
    """ Read gff file, convert to GenomeDict data.
        Only contain exon, gene now. Others should be added.
    """
    def __init__(self,infile,sep="\t",keeps=set(["exon","gene"]),fm="normal"):
        super(GffDict,self).__init__([])
        self.loadfile(infile,sep,keeps,fm)

    def loadfile(self,infile,sep,keeps,fm):
        for rec in Gff(infile,sep=sep,fm=fm):
            # filter type
            if (keeps != None) and (rec.type not in keeps):
                continue
            # Some illegle records.
            if (rec.gene_id is None or
                rec.tx_id is None):
                continue
            if rec.type == "exon":
                self.add_exon(EXON(rec))
            elif rec.type == "gene":
                self.add_gene_rec(rec)

    # Convert gffdict to {tx_id:tx_obj}
    def get_TxDict(self):
        data = Ordic()
        for mychr in self.values():
            for gene in mychr.values():
                for tx in gene.values():
                    data[tx.ID] = tx
        return data

    def _iter_chr_to_genes(self):
        for _chrid, _chr in self.items():
            _genes = _chr.values()
            yield _chrid, _genes

    def _iter_chr_to_txs(self):
        for _chrid, _genes in self._iter_chr_to_genes():
            _txs = []
            for _gene in _genes:
                _txs.extend(_gene.values())
            yield _chrid, _txs

    def get_chr_to_x(self,e_type,strand=False):
        """ Get a list of {chr:[transcripts/genes]}
            :param: e_type, element type, can be one of [tx, gene].
            :param: strand, set strand to true to include strand in
                key. The key will be (chr, strand) rather than chr.
        """
        if e_type == "tx":
            _iter_func = self._iter_chr_to_txs
        elif e_type == "gene":
            _iter_func = self._iter_chr_to_genes

        data = Ordic()

        # for the saving of time
        if not strand:
            for _chrid, _xs in _iter_func():
                data[_chrid] = _xs
            return data

        for _chrid, _xs in _iter_func():
            for _x in _xs:
                outkey = (_chrid, _x.strand) if strand else _chrid
                _vals = data.setdefault(outkey, [])
                _vals.append(_x)
        return data
            
    def get_chr_to_gene(self,if_strand=False):
        return self.get_chr_to_x("gene",if_strand)

    def get_chr_to_tx(self,if_strand=False):
        return self.get_chr_to_x("tx",if_strand)

    def conv_gffdict2tx_level(self):
        return self.get_TxDict()

class GtfDict(SGENT):
    
    """ Read gtf file, convert to GenomeDict data.
        Only exon are used.

        Example(Maybe correct):
        genomedict = GtfDict(input_gtf_file)
        # Get a geneDict object for gene named "EXMP"
        dict_of_gene = genomedict.get_GeneDict()
        gene = dict_of_gene.get("EXMP")
        # Get information of the gene
        print(gene.start)
        print(gene.end)
        print(gene.strand)
        # Length of gene (sum of exon length), might be wrong, Fix in next version
        print(len(gene))
        # How many transcript in a gene
        print(gene.count)

        to-do: Rename this object as Genome,
        and keep this object as alias of Genome

    """
    def __init__(self,infile,sep="\t",keeps=set(["exon"]),fm="normal"):
        """ fm[normal,gc,none]: format of gtf/gff file.
                normal, in most situation, normal format are suitable.
                gc, grass carp gtf V1.
                none, don't parse geneID and txID.
        """
        super(GtfDict,self).__init__([])
        self.loadfile(infile,sep,keeps,fm)

    def loadfile(self,infile,sep,keeps,fm):
        for rec in Gff(infile,sep=sep,fm=fm):
            # filter type
            #print(type(rec))
            if (keeps != None) and (rec.type not in keeps):
                continue
            # Some illegle records without 
            # gene_id or transcript_id attribution.
            if (rec.gene_id is None or
                rec.tx_id is None):
                continue
            self.add_exon(EXON(rec))

    def __len__(self):
        return self.get_count()

    def add_exon(self,exon):
        if exon.Chr in self.keys():
            self[exon.Chr].add_exon(exon)
        else:
            self.add_e(ChrDict(GeneDict(TxDict([exon]))))

    def add_e(self,gene):
        assert isinstance(gene,ChrDict)
        super(GtfDict,self).add_e(gene)

    def add_gene(self,gene):
        self.add_e(ChrDict(gene))

    def add_gene_rec(self,rec):
        gene = GeneDict(rec=rec)

    def check_strand(self,e):
        pass

    def get_GeneDict(self):
        """ Get a dict of gene_id - geneDict pair.
            {gene_id:GeneDict}
        """
        data = Ordic()
        for mychr in self.values():
            for gene in mychr.values():
                data[gene.ID] = gene
        return data

    # Convert gffdict to {tx_id:tx_obj}
    def get_TxDict(self):
        """ Get a dict of tx_id - txDict pair.
            {tx_id:TxDict}
        """
        data = Ordic()
        for gene in self.get_GeneDict().values():
            for tx in gene.values():
                data[tx.ID] = tx
        return data

    def rename_gene_tx(self, gene_id, tx_id = None, get_id = False):
        """ Rename the gene id with gene_id, transcript id with tx_id.
            When tx_id is not provide, name it after gene_id.
            example:
                1.
                gene_id = HBG
                tx_id = None
                xxxx ... transcript_id "HBG.1.1"; gene_id "HBG.1"
                xxxx ... transcript_id "HBG.1.2"; gene_id "HBG.1"
                xxxx ... transcript_id "HBG.2.1"; gene_id "HBG.2"
                2.
                gene_id = HBG
                tx_id = HBT
                xxxx ... transcript_id "HBT.1.1"; gene_id "HBG.1"
                xxxx ... transcript_id "HBT.1.2"; gene_id "HBG.1"
                xxxx ... transcript_id "HBT.2.1"; gene_id "HBG.2"
            Return a tuple (A, B)
            A is a dict, which is same as the return of get_GeneDict().
            B is a list, records old/new id.
            (old id, new id, "g"/"t") is the element of B, where "g" means gene id and "t" means transcript id.
            If get_id set to False, only return A.
        """
        data = Ordic()
        id_mapping = []
        if tx_id == None:
            tx_id = gene_id
        # 1-based results
        gene_count = 1
        geneDict = self.get_GeneDict()
        for ogid, txs in geneDict.items():
            # gene suffix
            gsf = "." + str(gene_count)
            gid = gene_id + gsf
            outgene = data.setdefault(gid,Ordic())
            id_mapping.append((ogid,gid,"g"))
            # 1-based results
            tx_count = 1
            for otid, exons in txs.items():
                # transcript suffix
                tsf = "." + str(tx_count)
                tid = tx_id + gsf + tsf
                outtx = outgene.setdefault(tid,Ordic())
                id_mapping.append((otid,tid,"t"))
                # 0-based exon-count
                exon_count = 0
                for exon in exons.values():
                    exon.edit_gene_id(gid)
                    exon.edit_tx_id(tid)
                    outtx[exon_count] = exon
                    exon_count += 1
                tx_count += 1
            gene_count += 1
        if get_id:
            return data, id_mapping
        else:
            return data
                    
# Get direction of two element
# "u" e1 locate in up (5')
# "d" e1 locate in down (3')
# "o" two element are overlaped.
def get_dir(st1,ed1,st2,ed2):
    def co(b1,a2,f = "u"):
        if b1 >= a2:
            return "o"
        else:
            return f
    if st1 < st2:
        return co(ed1,st2,"u")
    else:
        return co(ed2,st1,"d")

def get_dir_e(e1,e2):
    st1,ed1 = e1.start, e1.end
    st2,ed2 = e2.start, e2.end
    return get_dir(st1,ed1,st2,ed2)

def check_overlap(st1,ed1,st2,ed2):
    if get_dir(st1,ed1,st2,ed2) == "o":
        return True
    else:
        return False

# Abord 19/1/12
def check_overlap1(st1,ed1,st2,ed2):
    def co(b1,a2):
        if b1 >= a2:
            return True
        else:
            return False
    if st1 < st2:
        return co(ed1,st2)
    else:
        return co(ed2,st1)

def check_overlap_e(e1,e2):
    st1,ed1 = e1.start, e1.end
    st2,ed2 = e2.start, e2.end
    return check_overlap(st1,ed1,st2,ed2)

# Check list of elements overlap
def check_overlap_l(data):
    data = sorted(data,key=lambda x:x.start)
    for i in range(len(data)-1):
        if check_overlap_e(data[i],data[i+1]):
            #print("gene1,gene2 = %s, %s" % (data[i].ID,data[i+1].ID))
            #print("s1,e1,s2,e2 = %d, %d, %d, %d" % (data[i].start,data[i].end,data[i+1].start,data[i+1].end))
            return True
    return False

def open_file(infile):
    try:
        filename = infile
        fp = open(infile)
    except TypeError:
        filename = infile.name
        fp = infile
    return filename, fp
        
#class Dictsql(object):
#    """A dictionary based on sqlit database"""
#    def __init__(self, db = None, name = ""):
#        self.db = db
#        self.name = name

from _collections_abc import Mapping

class newGENT(object):
    """ Genomic element.
    """
    def __init__(self, *args, engine="dypylib", **kwargs):
        """Should make a function to ensure self.db = self.db,
        self.engine = self.engine when create from newGENT
        """
        if engine == 'gffutils':
            self._gffutils__init__(*args, **kwargs)

    def __getattr__(self, attr):
        # Try to return attributs definded in database.
        if self.engine == 'gffutils':
            return self._gffutils__getattr__(attr)

    def _gffutils__init__(self, db, *args, name = "",\
            **kwargs):
        self.engine = 'gffutils'
        self.db = db
        self.name = name

    def _gffutils__getattr__(self, attr):
        # Try to return attributs definded in database.
        return getattr(self.db[self.name], attr)

class BioMapping(Mapping, newGENT):
    """Mapping object for biology purpose.
    
    Data are deposited in database.
    to-do: Make this object a database version of GENT,
        or make a new GENT class (mGENT)which is child of BioMapping.

        get_parents -> get_parents_id like get_children_id
    """

    def __getitem__(self, key):
        if self.engine == 'gffutils':
            return self._gffutils__getitem__(key)

    def __contains__(self, key):
        return key in self.children_id

    def _gffutils__getattr__(self, attr):
        """Don't init children and parents in __int__,
        rather find it from datbase when is needed.

        This option can avoid loading every relation in
            python once construct an object.
        """
        if attr == 'children':
            self.children = self.get_children()
            return self.children
        elif attr == 'parents':
            self.children = self.get_parents()
            return self.children
        elif attr == 'children_id':
            self.children_id = self.get_children_id()
            return self.children_id
        else:
            return getattr(self.db[self.name], attr)

    def __iter__(self):
        """Self.keys() are generated here."""
        keys = self.children_id
        return iter(keys)

    def __len__(self):
        return len(self.children_id)

    def get_children_id(self):
        if self.engine == 'gffutils':
            return self._gffutils_get_children_id()
        else:
            raise TypeError("Only gffutils have get_children_id function")

    def _gffutils_get_children_id(self):
        children = self.db.children(self.name)
        return [i.id for i in children]

    def get_children(self):
        return self._gffutils_get_children()

    def _gffutils_get_children(self):
        return self.db.children(self.name)

    def get_parents(self):
        return self._gffutils_get_parents()

    def _gffutils_get_parents(self):
        return self.db.parents(self.name)

    def _gffutils__getitem__(self, key):
        if key not in self:
            info = "{} is not contained in {}, please check it.\
                    ".format(key, self.name)
            raise Keself.dbyError(info)
        return self.db[key]


class CDS(newGENT):
    pass
class Exon(newGENT):
    pass
class Transcript(BioMapping):
    """Chromosome object
    """

    def _gffutils__getitem__(self, key):
        if key not in self:
            info = "{} is not contained in {}, please check it.\
                    ".format(key, self.name)
            raise KeyError(info)
        feature = self.db[key]
        if feature.featuretype == 'exon':
            return Exon(self.db, name=feature.id, engine='gffutils')
        elif feature.featuretype == 'cds':
            return CDS(self.db, name=feature.id, engine='gffutils')
        else:
            return newGENT(self.db, name=feature.id, engine='gffutils')

class Gene(BioMapping):
    """Chromosome object
    """

    def _gffutils__getitem__(self, key):
        if key not in self:
            info = "{} is not contained in {}, please check it.\
                    ".format(key, self.name)
            raise KeyError(info)
        return Transcript(self.db, name=key, engine='gffutils')

    def _gffutils_get_children_id(self):
        children = []
        records = self.db.children(self.name, featuretype = 'transcript')
        for rec in records:
            children.append(rec.id)
        return children

class Chr(BioMapping):
    """Chromosome object
    """

    def _gffutils__getitem__(self, key):
        if key not in self:
            info = "{} is not contained in {}, please check it.\
                    ".format(key, self.name)
            raise KeyError(info)
        return Gene(self.db, name=key, engine='gffutils')

    def _gffutils_get_children_id(self):
        keys = []
        order = "select id from features where seqid = '%s' AND featuretype = 'gene'" \
                % self.name
        c = self.db.execute(order)
        for i in set(c.fetchall()):
            keys.append(i['id'])
        return keys

class Genome(BioMapping):
    """Genome object.
    
    The children can be chromosomes, schaffolds, or any other
    features previous defined in the annotation file.

    to-do: Rewrite SGENT.
    """

    def _gffutils__getitem__(self, key):
        if key not in self:
            info = "{} is not contained in {}, please check it.\
                    ".format(key, self.name)
            raise KeyError(info)
        return Chr(self.db, name=key, engine='gffutils')

    def _gffutils_get_children_id(self):
        keys = []
        c = self.db.execute("select DISTINCT seqid from features")
        for i in c.fetchall():
            keys.append(i['seqid'])
        return keys

    def convert_feature(self, feature):
        return self._gffutils_convert_feature(feature)

    def _gffutils_convert_feature(self, feature):
        """Convert gffutil object to dypylib object"""
        if feature.featuretype == 'gene':
            return Gene(self.db, name = feature.id, engine='gffutils')
        elif feature.featuretype == 'transcript':
            return Transcript(self.db, name = feature.id, engine='gffutils')
        elif feature.featuretype == 'exon':
            return Exon(self.db, name = feature.id, engine='gffutils')
        elif feature.featuretype == 'cds':
            return CDS(self.db, name = feature.id, engine='gffutils')
        else:
            return feature

    def search(self, key):
        return self._gffutils_search(key)
    
    def _gffutils_search(self, key):
        """Search by feature id.
        Except for chromosome, all other ids like gene/transcript id
            are stored at database.
        """
        # Return a chromsome
        if key in self:
            return self[key]
        else:
            return self.convert_feature(self.db[key])
    
def create_genome_using_gffutils(infile, dbfn = ':memory:'):
    """Create Genome object using gffutils
        1. Create gff_db
        2. Create Genome object with gff_db

        to-do: Write method for creating database, which would help
            to efficient create biology objects.
    """
    try:
        gff_db = gffutils.create_db(infile, dbfn)
    # gff_db has already created and saved in memory
    except sqlite3.OperationalError:
        gff_db = gffutils.interface.FeatureDB(dbfn)
    return create_genome_from_gffutils_db(gff_db)

def create_genome_from_gffutils_db(db):
    return Genome(db, engine = 'gffutils')

def test_genome_object(test_gtf):
    mygenome = create_genome_using_gffutils(test_gtf)
    print("")

    print(mygenome)

    print("Test keys():")
    for i in mygenome.keys():
        print(i)

    print("Test for-loop")
    for i in mygenome:
        print(i)

    print("Test items")
    for k,v in mygenome.items():
        print(k)
        print(v)



if __name__ == '__main__':

    import sys 
    test_gtf = "../../tests/data/test.gtf"
    test_genome_object(test_gtf)
    ###x = BioMapping(db = '111')
    ###for i in x:
    ### print(i)
    #gff_db = gffutils.create_db(test_gtf,':memory:')
    #gff_db = gffutils.interface.FeatureDB(':memory:')
    #c = gff_db.cursor()
    #c = gff_db.execute("select seqid from features")

    #for i in c.fetchall():
    #    print(i['seqid'])
    #print(gff_db.featchall())

    #print(gffutils.constants.SCHEMA)



#!/usr/bin/env python3

"""Sample fastq file.
Usage: cat input.fastq | ./subset_fastq.py - proportion > out.fastq
example: cat input.fastq | ./subset_fastq.py - 0.01 > out.fastq 
[Subset 1% reads of input.fastq to new file: out.fastq]
"""
import sys

def iter_fastq(infile):
    data = []
    for line in infile:
        if len(data) == 4:
            yield data
            data = []
        data.append(line)

def conv_proportion_to_skip_number(num):
    ''' Convert the subseting proportion to the number of records
        to skip between two output records.
    '''
    if num > 1:
        sys.exit("The proportion must be smaller than 1. While {0} is given.\n".format(num))
    num_skip = int(1/num - 1)
    return num_skip

def main(argv):
    import argparse
    parser = argparse.ArgumentParser(\
    formatter_class = argparse.RawDescriptionHelpFormatter,\
    description = __doc__)

    parser.add_argument('infile',nargs='?',help="Input fastq file, \"-\" for stdin ",type=argparse.FileType('r'))
    parser.add_argument('-o','--outfile',nargs='?',help="output file",default=sys.stdout,type=argparse.FileType('w'))
    parser.add_argument('-p','--proportion',nargs='?',help="Proportion of reads to be output",type=float)
    args = parser.parse_args(argv[1:])

    # ns: number of records to skip
    ns = conv_proportion_to_skip_number(args.proportion)
    fq_iterator = iter_fastq(args.infile)
    # cycle: 0,1,2,3,4,....1+ns
    # The 0-th records will be output
    # And the other records is omitted.
    count = 0
    for data in fq_iterator:
        count += 1
        if count == 1:
            args.outfile.write("".join(data))
        if count == ns+1:
            count = 0

if __name__ == '__main__':

    main(sys.argv)



#!/usr/bin/env python

'''
This Script is used for parsing GBFF.

_author:You-Duan
_version:0.0.3

'''
import re, sys

def nonepop(mylist):#del last item when its none('')

    if mylist[-1]:
        return mylist
    mylist.pop()
    return mylist
    
def unquota(mystring):

    if mystring[0] == "\"":
        mystring = mystring[1:]
    if mystring[-1] == "\"":
        mystring = mystring[:-1]
    return mystring
    
class Replace(object):

    '''read and save replace file
    
       data:#the data structure is too complex!!!
       {feature1:
       {(subkey1,subkey2,...):(value1,value2,value3,...),
        (subkey1,subkey2,...):(v1,v2,v3,...),...}
        feature2:{},featrue3:{},...}
    '''
    '''can only support recognize and replace in the same feature!!!
    '''
    def __init__(self,replace_file):
    
        self.data = {}
        self.key_names = []#names determine if the feature is finded
        self.target_names = []#names to be changed
        self.load_replace_file(replace_file)
        
    def load_replace_file(self,file,sep="\t"):

        '''replace_file:
           subkey1;subkey2;subkey3;    replace1;replace2;replace3;    feature
           ..;..;..;..;    ..;..;..;    feature2
           ..;..;..;..;    ..;..;..;    feature
           ......
        '''
        firstline = file.readline().split(sep)
        self.key_names = firstline[0].split(";")
        self.target_names = firstline[1].split(";")
        nonepop(self.key_names)
        nonepop(self.target_names)
        for eachline in file:
            tmp = eachline.strip().split(sep)
            area1 = tuple(nonepop(tmp[0].split(";")))
            area2 = tuple(nonepop(tmp[1].split(";")))
            try:
                feature = tmp[2]
            except:
                pass
            self.data.setdefault(feature,{})
            self.data[feature][area1] = area2
            
class Gb_Feature(object):

    '''read and save genebank feature
       
       Public:
       locPattern = location pattern
       eg. "CDS             4071..4523"
       desPattern = description pattern
       eg. "/product="tRNA-Glu""
       deslocPattern = description including location pattern
       eg. "/anticodon=(pos:6702..6704,aa:Gly,seq:tcc)"
       deslocPattern2
       eg. "/transl_except=(pos:6962,aa:TERM)"
       namePattern = ORGANISM name
       eg. "  ORGANISM  Tetrancistrum nebulosi"
       locRdec = description related to location; type:list.
       
       Private:
       fea = feature name; type:string.
       locst = start location; type:int.
       loced = end location; type:int.
       qua = qualifiers; type:dic.
       qua = [key1:[value1,value2,...],key2:[value1,value2,...],...]
       quakeys = qua keys.
    '''
    
    namePattern = re.compile(r'\s{2}ORGANISM\s*(.*)')
    # issue:
    # 1.
    #     tRNA            complement(1064049..1064125)
    # 2.
    #      CDS             complement(join(1310321..1311190,1311192..1311311,
    #                 1311315..1311491))

    locPattern = re.compile(r'\s{5}(\w+)\s+.*')
    locPattern0 = re.compile(r'(\w+)\s+(\d+)\.\.(\d+)\s?')
    # back-up property.
    locPattern0000 = re.compile(r'\s{5}(\w+)\s+(\d+)\.\.(\d+)\s?')
    desPattern = re.compile(r'\s{10,}/(\w+)=(.*)\n')
    desPattern0 = re.compile(r'/(\w+)=(.*)')
    deslocPattern = re.compile(r'(\D+)(\d+)\.\.(\d+)(\D+)')
    deslocPattern2 = re.compile(r'(\D+pos:)(\d+)(\D+)')
    locRdec = []
    
    def __init__(self,file):
    
        self.fea = ""
        self.locst = 1
        self.loced = 1
        self.qua = {}
        self.quakeys = []
        self.loadfile(file)
        
    def get(self,key):
    
        return self.qua.get(key)

    def init_header(self,line):
        tmp = Gb_Feature.locPattern0.match(line)
        self.fea = line.split()[0]
        if tmp:#first line
            self.locst = int(tmp.group(2))
            self.loced = int(tmp.group(3))
        else:
            # not the typical start and end.
            # Don't record.
            pass

    def init_qualifier(self,line):
        tmp = Gb_Feature.desPattern0.match(line)
        if tmp.group(1) not in self.quakeys:
            self.quakeys.append(tmp.group(1))
        #self.qua[tmp.group(1)] = tmp.group(2)
        self.qua.setdefault(tmp.group(1),[]).append(tmp.group(2))
        if tmp.group(2)[0] == "(" and tmp.group(1) not in \
        Gb_Feature.locRdec:#contain position information
            Gb_Feature.locRdec.append(tmp.group(1))

    def init_block(self,data,flag):
        data = "".join(data)
        if flag:
            self.init_qualifier(data)
        else:
            self.init_header(data)

    def loadfile(self,file):
        data = []
        header_loaded = False
        for each in file:
            tmp = Gb_Feature.desPattern.match(each)
            if tmp:#qualifier line
                self.init_block(data, header_loaded)
                data = []
                # The first line is always the location line, therefor,
                # whenever inited, the location must have loaded.
                header_loaded = True
            data.append(each.strip())
        self.init_block(data, header_loaded) 
        
    def change_qua(self,qua_names,replace_dic,rep_names):
    #change qualifiers according to replace_dic
    #qua_names:list or tuple. determine which match situation it is,
    #corresponding to area1 of firstline of replace file.
    #rep_names:list or tuple. determine which items to be replaced,
    #corresponding to area2 of firstline of replace file.
    
        if self.fea not in replace_dic:#none of this feature's bussiness
            return
        my_rep_dic = replace_dic.get(self.fea)
        #my_rep_dic = {
        #(subkey1,subkey2,...):(value1,value2,value3),
        #(subkey1,subkey2,...):(v1,v2,v3,...),
        #...,...,...}
        test_keys = []
        if isinstance(qua_names,str):
            qua_names = [qua_names]
        for each in qua_names:
            try:
                test_keys.append(self.qua.get(each)[-1])#needs fix!!!
            except:
                pass
        tmp = self.mapkeys(test_keys,my_rep_dic)
        if tmp:#Successful matches
            values = my_rep_dic.get(tmp)
            for i in range(len(values)):
                if rep_names[i] in self.qua:#if key not exist,do not creat it.
                    self.qua[rep_names[i]][-1] = "\""+unquota(values[i])+"\""
            
    def mapkeys(self,testkey,mykeys):
    #if testkey in mykeys(to our rules):
    #return the key in mykeys.
    #testkey = (subkey1,subkey2,subkey3) can't be key
    #mykeys = [testkey1,testkey2,testkey3,...]
    #our rules:none type in mykeys matchs every key


        def judge(a,b):#judge subkey
        
            if not b:#b == ''
                return True
            if a == b:
                return True
            return False
        for each in mykeys:
            if not (isinstance(each,list) or isinstance(each,tuple)):
                print("[Error:] please contact the author to solve it.")
                sys.exit()
            for i in range(len(each)):
                if not judge(unquota(testkey[i]),each[i]):
                    break
                if i == len(each)-1:#every testkey[i] matchs each[i]
                    return each
        return None
        
class Genebank_parser(object):
    
    '''read and save genebank file.
       data:
       hddat = head data (never mind); type:list; eachline -> eachelement.
       ftlin = features line; type:string.
       mddat = mid data (features); type:list.
       tldat = tail data (sequence); type:string.
    '''
    
    def __init__(self,file):
    
        self.hddat = []
        self.ftlin = ""
        self.mddat = []
        self.tldat = ""
        self.loadfile(file)
        self.length = len(self.tldat)
        
    def __len__(self):
    
        return self.length
    
    def loadfile(self,file):

        def loadpart(part,file):#decide which part to be loaded
            if part == "A":
                self.loadhdpart(file)
            if part == "B":
                self.loadmdpart(file)
            if part == "C":
                self.loadtlpart(file)
                
        tmpfile = []
        partchanged = False
        currPart = "A"
        for each in file:
            if each[0:8] == "FEATURES":
                nextPart = "B"
                partchanged = True
            if each[0:6] == "ORIGIN":
                nextPart = "C"
                partchanged = True
            if partchanged:
                loadpart(currPart,tmpfile)
                tmpfile = [each]
                partchanged = False
                currPart = nextPart
                continue
            tmpfile.append(each)
        loadpart(currPart,tmpfile)
        
    def loadhdpart(self,file):
        
        for each in file:
            self.hddat.append(each.rstrip())
            
    def loadmdpart(self,file):
        
        self.ftlin = file[0].rstrip()
        currqual = [file[1]]
        for each in file[2:]:
            if Gb_Feature.locPattern.match(each):#location part
                self.mddat.append(Gb_Feature(currqual))
                currqual = [each]
            currqual.append(each)
        self.mddat.append(Gb_Feature(currqual))
            
    def loadtlpart(self,file):
    
        _data = []
        for each in file[1:]:
            tmp = each.strip().split()
            # end of file
            if tmp[0] == "//":
                self.tldat = "".join(_data)
                return
            # The first element of tmp is not sequence.
            # tmp: ['1649941', 'ggcgttaagg', 'cctgcaaaca', 
            # 'ccacgcaggc', 'ttgcggtgcc', 'tgggcatcgg', 
            # 'ctgcagcttc']
            _data.extend(tmp[1:])
                
    def sortmddat(self):
    
        self.mddat = self.mysort(self.mddat,True,False)
        #first sort according to start location
        same_st_indx = currst = 0
        #same_st_indx:start index of the same locations
        #currst = current start location
        currst = self.mddat[0].locst
        for i in range(len(self.mddat)):#seconde sort according to end location
            #apply it only when start locations are same
            if self.mddat[i].locst != currst:#comes to the next start location
                if same_st_indx != i-1:#start locations not unique
                    self.mddat[same_st_indx:i] = \
                    self.mysort(self.mddat[same_st_indx:i],False,True)
                same_st_indx = i
                
    def mysort(self,data,isstart=True,descending=False):
    
        length = len(data)
        if isstart:
            whichloc = ".locst"
        else:
            whichloc = ".loced"
        if descending:
            whichcom = "<"
        else:
            whichcom = ">"
        for i in range(length):
            for j in range(length-i-1):
                if eval("data[j]"+whichloc+whichcom+"data[j+1]"+whichloc):
                    data[j],data[j+1] = data[j+1],data[j]
        return data
        
    def set_site(self,old_site,to_site):
    
        self.set_start(int(old_site)-int(to_site)+1)
        
    def set_start_tl(self,old):
    
        #set old site to 1
        #change the tl part
        self.tldat = self.tldat[old-1:]+self.tldat[:old-1]
        
    def set_start(self,old):
    
        #set old site to 1
        def myfun(loc):#ensure 1 < loc <= length
            if 0 < loc <= self.length:
                return loc
            if loc > self.length:
                return myfun(loc - self.length)
            return (self.length + loc)
        
        old = int(old)
        for each in self.mddat:#each = one Gb_Feature;some times rename it
            if each.fea == "source":
                continue
            each.locst, each.loced = \
            myfun(each.locst-old+1), myfun(each.loced-old+1)
            for i in each.quakeys:#set qualifiers
                if i in Gb_Feature.locRdec:#needs to be repaired!!!
                    tmp = Gb_Feature.deslocPattern.match(each.qua[i][-1])
                    if tmp:
                        tmp = tmp.groups()
                        each.qua[i][-1] = tmp[0]+str(myfun(int(tmp[1])-old+1))\
                        +".."+str(myfun(int(tmp[2])-old+1))+tmp[-1]
                        continue
                    tmp = Gb_Feature.deslocPattern2.match(each.qua[i][-1])
                    if tmp:
                        tmp = tmp.groups()
                        each.qua[i][-1] = tmp[0]+str(myfun(int(tmp[1])-old+1))\
                        +tmp[-1]
                        continue
        self.set_start_tl(old)
        
    def change_qua(self,file):
    
        chgEx = Replace(file)#changeExample
        for each in self.mddat:
            each.change_qua(chgEx.key_names,chgEx.data,chgEx.target_names)
            
    def get_features(self,feature_names=[],filt_feas=[]):
    #get all featrues which name is in feature_names 
    #not in filt_feas and return in a list
    #if feature_names == []:all featrues is needed
    #if filt_feas == []:do not filter
    
        myfeatures = []
        if isinstance(feature_names,str):
            feature_names = [feature_names]
        if isinstance(filt_feas,str):
            filt_feas = [filt_feas]
        tmp = ''
        for each in self.mddat:
            if not feature_names:#all features take into account
                if each.fea not in filt_feas:
                    myfeatures.append(each)
                continue
            if each.fea in feature_names and each.fea not in filt_feas:
                myfeatures.append(each)
        return myfeatures
        
    def get_qualifiers(self,qua_names,fea_names=[],filt_feas=[]):
    
        myquas = []
        if isinstance(qua_names,str):
            qua_names = [qua_names]
        for each in self.get_features(fea_names,filt_feas):#every feature
            for qua_name in qua_names:#return qua_name is first finded
                tmp = each.get(qua_name)
                if tmp:
                    myquas.extend(tmp)
                    break
        return myquas
            
    def writeGBFF(self,file):
    
        #print hddat
        for each in self.hddat:
            file.write(each+"\n")
        #print mddat
        file.write(self.ftlin+"\n")
        for each in self.mddat:
            file.write("     %-16s%d..%d"%(each.fea,each.locst,each.loced)+"\n")
            for eachkey in each.quakeys:
                for i in each.qua[eachkey]:
                    text = "/%s=%s"%(eachkey,i)
                    file.write(("%21s"%"")+("\n%21s"%"").join(text[j*58:(j+1)*58] \
                    for j in range(len(text)//58+1))+"\n")
        #print tldat
        file.write("ORIGIN\n")
        for i in range(0,len(self.tldat),60):
            tmp = self.tldat[i:i+60]
            file.write("%9d "%(i+1))
            file.write(" ".join(tmp[j*10:(j+1)*10] for j in \
            range(min((len(tmp)//10+1),6)))+"\n")
        file.write("//\n")

if __name__ == '__main__':

    import sys, argparse, textwrap
    
    parser = argparse.ArgumentParser(\
        formatter_class=argparse.RawDescriptionHelpFormatter,\
        prog="gbParser",\
        description="Change Ori_site to New_site",\
        epilog=textwrap.dedent(\
        '''example :
            1.python gbParser.py myGBFF.gb -s 50 -o newGBFF.gb [change site]
            2.python gbParser.py -s 50 -n 3 -o newGBFF.gb myGBFF.gb 
            3.python gbParser.py myGBFF.gb [nothing changed]
            4.python gbParser.py myGBFF.gb -r replace.txt [only replace features]
            5.python gbParser.py myGBFF.gb -g [output a file used to build a tree]
        '''))
    parser.add_argument('infile',help="GBFF file",type=argparse.FileType('r'))
    parser.add_argument('-s','--ori_site',help="the site to be changed",type=int)
    parser.add_argument('-n','--new_site',help="the new site",default=1,type=int)
    parser.add_argument('-r','--rep_file',help="replace file",type=argparse.FileType('r'))
    parser.add_argument('-o','--out_file',help="output gb file [default:stdout]",nargs='?',\
    const=sys.stdout,type=argparse.FileType('w'))
    parser.add_argument('-t','--tree',help="output tree building file [default:tree.txt]",\
    nargs='?',const="tree.txt",type=argparse.FileType('w'))
    myargs = parser.parse_args(sys.argv[1:])
    
    def write_tree(mygb,outfile):
    
        myqua_names = ["gene","product"]
        myfilter = ["CDS"]
        for each in mygb.hddat:#write name 
            tmp = Gb_Feature.namePattern.match(each.rstrip())
            if tmp:
                outfile.write(">"+"_".join(tmp.group(1).split())+"\n")
                break
        outfile.write(" ".join(map(unquota,\
        mygb.get_qualifiers(myqua_names,filt_feas=myfilter))))
    
    def main():
        mygb = Genebank_parser(myargs.infile)
        if myargs.ori_site:
            if myargs.ori_site < 1 or myargs.ori_site > len(mygb):
                print("[Error]:the ori_site you offered is wrong(too big or small),\
try another site")
                sys.exit()
            mygb.set_site(myargs.ori_site,myargs.new_site)
            mygb.sortmddat()
        if myargs.rep_file:
            mygb.change_qua(myargs.rep_file)
        if myargs.tree:
            write_tree(mygb,myargs.tree)
        if myargs.out_file:
            mygb.writeGBFF(myargs.out_file)
    main()
    



#!/usr/bin/env python

NCBI2CHR_hs38={"NC_000010.11":"chr10","NC_000011.10":"chr11","NC_000012.12":"chr12","NC_000013.11":"chr13","NC_000014.9":"chr14","NC_000015.10":"chr15","NC_000016.10":"chr16","NC_000017.11":"chr17","NC_000018.10":"chr18","NC_000019.10":"chr19","NC_000001.11":"chr1","NC_000020.11":"chr20","NC_000021.9":"chr21","NC_000022.11":"chr22","NC_000002.12":"chr2","NC_000003.12":"chr3","NC_000004.12":"chr4","NC_000005.10":"chr5","NC_000006.12":"chr6","NC_000007.14":"chr7","NC_000008.11":"chr8","NC_000009.12":"chr9","NC_012920.1":"chrM","NC_000023.11":"chrX","NC_000024.10":"chrY"}
NCBI2CHR_mm10={"NC_000076.6":"chr10","NC_000077.6":"chr11","NC_000078.6":"chr12","NC_000079.6":"chr13","NC_000080.6":"chr14","NC_000081.6":"chr15","NC_000082.6":"chr16","NC_000083.6":"chr17","NC_000084.6":"chr18","NC_000085.6":"chr19","NC_000067.6":"chr1","NC_000068.7":"chr2","NC_000069.6":"chr3","NC_000070.6":"chr4","NC_000071.6":"chr5","NC_000072.6":"chr6","NC_000073.6":"chr7","NC_000074.6":"chr8","NC_000075.6":"chr9","NC_005089.1":"chrMT","NC_000086.7":"chrX","NC_000087.7":"chrY",}
NCBI2CHR_Gz10={"NC_007121.6":"chr10","NC_007122.6":"chr11","NC_007123.6":"chr12","NC_007124.6":"chr13","NC_007125.6":"chr14","NC_007126.6":"chr15","NC_007127.6":"chr16","NC_007128.6":"chr17","NC_007129.6":"chr18","NC_007130.6":"chr19","NC_007112.6":"chr1","NC_007131.6":"chr20","NC_007132.6":"chr21","NC_007133.6":"chr22","NC_007134.6":"chr23","NC_007135.6":"chr24","NC_007136.6":"chr25","NC_007113.6":"chr2","NC_007114.6":"chr3","NC_007115.6":"chr4","NC_007116.6":"chr5","NC_007117.6":"chr6","NC_007118.6":"chr7","NC_007119.6":"chr8","NC_007120.6":"chr9","NC_002333.2":"chrM"}#GRCz10

def gff_ncbi2chr(infile,outfile,nmDict,sep="\t",filt_chr=False):
    
    try:infile=open(infile)
    except:pass
    try:outfile=open(outfile,'w')
    except:pass

    for line in infile:
        tmp = line.strip().split(sep)
        _ = nmDict.get(tmp[0])
        if filt_chr and not _:
            continue
        tmp[0] = nmDict.get(tmp[0]) or tmp[0]
        outfile.write(sep.join(tmp)+"\n")

def fa_ncbi2chr(infile,outfile,nmDict,sep="\t"):
    #used once. for further ultilize some update must be applied
    try:infile=open(infile)
    except:pass
    try:outfile=open(outfile,'w')
    except:pass

    for line in infile:
        if line[0] == ">":
            line_array=line.strip().split()
            name=line_array[0][1:]
            name=nmDict.get(name) or name
            outfile.write(">"+name+"\n")
        else:
            outfile.write(line)

def fa_ncbi2chr2(infile,outfile,nmDict,sep="\t"):
    #used once.for refSeq genome as gi|xxx|ref|xxxxx| other information
    #for further ultilize some update must be applied
    try:infile=open(infile)
    except:pass
    try:outfile=open(outfile,'w')
    except:pass

    for line in infile:
        if line[0] == ">":
            line_array=line.strip().split()
            name=line_array[0][1:]
            ref_name_array = name.split("|")
            ref_name = ref_name_array[3]
            name=nmDict.get(ref_name) or name
            outfile.write(">"+name+"\n")
        else:
            outfile.write(line)
#import sys
#gff_ncbi2chr(sys.argv[1],sys.stdout)

def main(argv):
    
    import argparse, sys

    parser = argparse.ArgumentParser(description="transform chromsome id from NCBI accssion number to chr* in gtf file")
    parser.add_argument('infile',nargs='?',default=sys.stdin,help="gtf/gff file",type=argparse.FileType('r'))
    parser.add_argument('-o','--outfile',nargs='?',help="output file",default=sys.stdout,type=argparse.FileType('w'))
    parser.add_argument('--sep',nargs='?',default="\t",help="default tab")
    parser.add_argument('--strict',action='store_true',help="filt out non-chromosome sequence")
    parser.add_argument('-s','--species',nargs='?',default="hs38",help="genome version",choices=["hs38","mm10","Gz10"])
    args = parser.parse_args(argv[1:])

    if len(argv) == 1:
        parser.print_help()
        sys.exit(1)
    if args.species == "hs38":
        gff_ncbi2chr(args.infile,args.outfile,NCBI2CHR_hs38,filt_chr=args.strict)
        #fa_ncbi2chr2(args.infile,args.outfile,NCBI2CHR_hs38)
    elif args.species == "mm10":
        gff_ncbi2chr(args.infile,args.outfile,NCBI2CHR_mm10,filt_chr=args.strict)
    elif args.species == "Gz10":
        #fa_ncbi2chr(args.infile,args.outfile,NCBI2CHR_Gz10)
        gff_ncbi2chr(args.infile,args.outfile,NCBI2CHR_Gz10,filt_chr=args.strict)
    else:
        raise NameError("you specify one unsupport genome version")

if __name__ == '__main__':

    import sys
    sys.exit(main(sys.argv))



#!/usr/bin/env python 

'''
[usage] ./filter file1 file2 outfile
    ./filter file1 file2 > outfile
[purpose]
    get lines in file1 which has key words in file2
'''

import sys

class CaseInsensitiveSet(set):
    """Make sense when element is tuple
    Refers to https://stackoverflow.com/questions/27531211/how-to-get-case-insensitive-python-set"""

    def get_lower(self, item):
        """Get lower for string or tuple"""
        if isinstance(item, str):
            return item.lower()
        elif isinstance(item, tuple):
            return tuple([self.get_lower(i) for i in item])
        else:
            return item

    def add(self, item):
        set.add(self, self.get_lower(item))

    def __contains__(self, item):
        return set.__contains__(self, self.get_lower(item))

def get_vals(lst,keycols):
    """ eg. get_vals([a,b,c,d], [0,3]) -> [a,d] """
    return tuple(map(lambda x:lst[x],keycols))

def readKeys(infile, sep, has_header=True, keycol=[0], ignorecase=False):
    if ignorecase:
        keys = CaseInsensitiveSet()
    else:
        keys = set()
    if has_header:
        infile.readline()
    for eachline in infile:
        tmp = eachline.strip("\n").split(sep)
        keys.add(get_vals(tmp, keycol))
    return keys

def _filter(infile,save_keys,sep,has_header,keycol,outfile,mode):
    """ Core function.
    mode:{filter,differ}
    mode_filter: return filter results
    mode_differ: return differ results
    """
    if has_header:
        outfile.write(infile.readline())
    #print(save_keys)
    for eachline in infile:
        tmp = eachline.strip("\n").split(sep)
        if (get_vals(tmp,keycol) in save_keys) and mode=="filter":
            outfile.write(eachline)
        elif (get_vals(tmp,keycol) not in save_keys) and mode=="differ":
            outfile.write(eachline)

def filter(infile,save_keys,sep,has_header,keycol,outfile):
    _filter(infile,save_keys,sep,has_header,keycol,outfile,"filter")

def differ(infile,save_keys,sep,has_header,keycol,outfile):
    _filter(infile,save_keys,sep,has_header,keycol,outfile,"differ")

def parse_keycol(keycol, sep=","):
    """ convert \"1,2,3,5\" into [1,2,3,5] """
    return list(map(int,keycol.strip().split(sep)))

def main(argv):
    import argparse
    parser = argparse.ArgumentParser(description="Filter file1 accoding to one file2. Two mode are avaliable.")
    parser.add_argument('file1',nargs='?',help="file to be filtered, \"-\" for stdin ")
    parser.add_argument('file2',nargs='?',help="file to guide the filtering, \"-\" for stdin(conflict with \"-\" for file1)")
    parser.add_argument('-o','--outfile',nargs='?',help="output file",default=sys.stdout,type=argparse.FileType('w'))
    parser.add_argument('-s1','--sep1',nargs='?',default="\t")
    parser.add_argument('-s2','--sep2',nargs='?',default="\t")
    parser.add_argument('-k1','--keycol1',nargs='?',default="0")
    parser.add_argument('-k2','--keycol2',nargs='?',default="0")
    parser.add_argument('-n1','--has_header1',action='store_false',help="if file1 hasn't header,set this option")
    parser.add_argument('-n2','--has_header2',action='store_false',help='same with n1 but for file2')
    parser.add_argument('-i','--ignorecase',action='store_true',help='Ignore case')
    parser.add_argument('-m','--mode',nargs='?',default='filter',choices=['filter','differ'])
    args = parser.parse_args(argv[1:])

    sep1=args.sep1
    sep2=args.sep2
    keycol1=parse_keycol(args.keycol1)#key column which define filter limits
    keycol2=parse_keycol(args.keycol2)
    has_header1=args.has_header1
    has_header2=args.has_header2
    
    if args.file1 == '-' and args.file2 == '-':
        raise KeyError("You can not set file1 and file2 to \"-\" at the same time!")
    if args.file1 == '-':
        file1=sys.stdin
    else:
        file1=open(args.file1)
    if args.file2 == '-':
        file2=sys.stdin
    else:
        file2=open(args.file2)
    outfile=args.outfile

    keys = readKeys(file2,sep2,has_header2,keycol2,args.ignorecase)
    if args.mode == "filter":
        filter(file1,keys,sep1,has_header1,keycol1,outfile)
    elif args.mode == "differ":
        differ(file1,keys,sep1,has_header1,keycol1,outfile)

if __name__ == '__main__':

    main(sys.argv)



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

# Update version from VERSION file into module
with open('VERSION', 'r') as fversion:
    version = fversion.readline().rstrip()
with open('_version.py', 'wt') as fversion:
    fversion.write('__version__ = "'+version+'"')

name = 'dypylib'
setuptools.setup(
    name=name, # Replace with your own username
    version=version,
    author='You Duan',
    author_email='duanyou@outlook.com',
    description='genomics research',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/yduanBioinfo/dypylib',
    install_requires = ['case-insensitive-dictionary'],
    #packages=setuptools.find_packages(),
    package_dir={name: '.'},
    packages=[name] + ['.'.join((name, x)) for x in setuptools.find_packages()],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
    ],
    #data_files = [
    #    ('',['VERSION'])
    #],
    scripts=[
        'scripts/subset_fasta.py',
        'scripts/subset_fastq.py',
        'scripts/annot_from_file.py',
        'scripts/filter.py',
        'bio/seq/plotsts.py',
        'bio/seq/ncbi_chr.py',
        'bio/seq/gtf2table.py',
        'bio/seq/countgtf.py',
        'bio/seq/gtf2stat.py',
        'bio/seq/format/merge_overlap_gene.py',
    ],
    python_requires='>=3.6',
)



#!/usr/bin/env python3

import sys
from dypylib.bio.seq.base import Gff, GtfDict

'''
input gtf should only contain exon feature.
stat for each transcript
TranscriptID,GeneID,length,count_of_exons,length_of_exons,count_of_introns,leng_of_introns
'''

def iter_stats(gtfdict):
    """ Get tx stats from GtfDict
    """
    txdict = gtfdict.get_TxDict()
    for key, val in txdict.items():
        tx_id = key
        gene_id = val.gene_id
        length = val.get_length()
        count_of_exons = val.get_count()
        length_of_exons = list(map(lambda x:len(x),val.values()))
        count_of_introns = count_of_exons - 1
        length_of_introns = list(map(lambda x:len(x),val.get_introns()))
        yield tx_id, gene_id, length, count_of_exons, length_of_exons, count_of_introns, length_of_introns

def iter_stats_str(gtfdict):
    """ Concate list element with ","
    """
    def _join(lst,sep=","):
        return sep.join(map(str,lst))
    for ss in iter_stats(gtfdict):
        res = list(ss)
        res[4] = _join(res[4])
        res[6] = _join(res[6])
        res = map(str,res)
        yield res

def gtf2table(gtfdict,outfile,write_header=None,sep="\t"):
    header = ["TranscriptID","GeneID","length","count_of_exons","length_of_exons","count_of_introns","length_of_introns"]
    if write_header:
        outfile.write(sep.join(header)+"\n")
    for line in iter_stats_str(gtfdict):
        outfile.write(sep.join(line)+"\n")

def main(argv):
    import argparse
    parser = argparse.ArgumentParser(description="Get a table description for gtf file.")
    parser.add_argument('infile',nargs='?',help="input gtf file",type=argparse.FileType('r'))
    parser.add_argument('-o','--outfile',nargs='?',help="outfile.",type=argparse.FileType('w'),default=sys.stdout)
    parser.add_argument('-t','--gff-type',nargs='?',default='normal',choices=['normal','gc'])
    parser.add_argument('-n','--none-header',action='store_false')
    args = parser.parse_args(argv[1:])

    mygtfdict = GtfDict(args.infile,fm=args.gff_type)
    gtf2table(mygtfdict,args.outfile,write_header=args.none_header)

if __name__ == '__main__':
    import sys
    main(sys.argv) 



#!/usr/bin/env python3

"""A collections of basic classes and methods.

The tools for reading/writing files are defined in this collection.

    Typical usage example:

    mydata = LineFileSpliter(infile, sep="\t", comment="#")
    for l in mydata:
        print(l[0])
"""
import sys
import logging

class BaseFile(object):
    """Basic class of File handle
    
    Attributes:
        filename:
          A string of filename.
        
    """

    def __init__(self, filename):
        """
        Args:
            filename:
              A string of filename.
        """
        self.filename = filename
        if filename:
            logging.debug("Load file `{0}`".format(filename))

class LineFile(BaseFile, list):
    """Generic file parser for line-based files
    """
    def __new__(self,*args,**kwargs):
        return list.__new__(self)

    def __init__(self, filename, comment=None, load=False):
        super(LineFile, self).__init__(filename)
        if load:
            fp = must_open(filename)
            #self.lines = [l.strip() for l in fp if l[0]!=comment]
            for l in fp:
                if l[0]!=comment:
                    self.append(l.strip())
            logging.debug("Load {0} lines from `{1}`.".\
                        format(len(self), filename))

class ListFile(LineFile):

    def __init__(self, *args, **kwargs):
        super(ListFile, self).__init__(*args, **kwargs, load=True)

class LineFileIterator(LineFile):
    """Iterator for line file.
    """
    def __init__(self, filename, comment=None, has_header=False):

        super(LineFileIterator,self).__init__(filename,comment,load=False)
        self.has_header = has_header
        self.fp = must_open(self.filename)
        self.comment = comment

        if has_header:
            self._readheader()

    def __iter__(self):

        return self.next()

    def _readheader(self):

        for line in self.fp:
            if self.is_comment(line):
                continue
            else:
                self._header = line.rstrip("\n")
                break

    def next(self):

        return self._iterator()

    def _iterator(self):
        
        for line in self.fp:
            line = line.rstrip("\n")
            if self.is_comment(line):
                continue
            else:
                yield line

    def is_comment(self,line):

        #Skip all line, when no comment defined.
        if not self.is_comment:
            return False
        if line[0] == self.comment:
            return True
        else:
            return False

    @property
    def header(self):

        if not self.has_header:
            raise TypeError("While has_header has set to False, can not return file header.\n")
        return self._header

class LineFileSpliter(LineFileIterator):
    """Iterator for line file.
    
    Read and convert a file to an iterator. Eachline will be split 
    into list.

    Attributes:
        _header: String to keep the head line.
    """

    def __init__(self,filename, sep="\t", comment=None, has_header=False):
        """
        Args:
            filename: Name of input file.
            sep: Seperater that used to split line into list. 
            comment: The line start with the comment string will be
              ignored. Typically, when comment is set to None, every
              line will keep.
            has_header: Whether the file has head line. True to skip
              header, and keep header into _header.
        """
        self.__sep = sep
        super(LineFileSpliter,self).__init__(filename, comment, has_header)

    def _readheader(self):

        super(LineFileSpliter,self)._readheader()
        self._header = self._header.split(self.__sep)

    def _iterator(self):

        for line in self.fp:
            line = line.rstrip("\n")
            if self.is_comment(line):
                continue
            else:
                yield line.split(self.__sep)

class DictFile(BaseFile, dict):
    """
    Generic file parser for multi-column files, keyed by a particular index.
    If valuepos is a single number, DictFile would be {key,value}.
    If valuepos is a list, DictFile would be {key,[values]}.
    """
    def __init__(self, filename, keypos=0, valuepos=1, delimiter=None,
                       strict=True, keycast=None, cast=None, has_header=False):
        super(DictFile, self).__init__(filename)

        # Load attribute.
        self._fp = must_open(filename)
        self.filename = filename
        self.strict = strict
        self.has_header = has_header
        self.__valuepos = self.__value2list(valuepos)#for muti-value purpose
        #ncols correct it self while reading lines.[Hidden bugs]
        # An error is going to triggered when valuepos is None.
        self.ncols = max(keypos, self._int(self.max(self.__valuepos))) + 1
        #if has_header:
        #    self._read_header(delimiter)
        self._read_header(delimiter)
        self._read_body(keypos,delimiter,keycast,cast)

    def _int(self, s):
        if s == None:
            return 0
        else:
            return int(s)

    def _read_header(self,delimiter):
        if self.has_header == False:
            return
        self.header = self._fp.readline().rstrip("\n")
        self.header_lst = self.header.split(delimiter)
        # Only keep correct field header accoding to valuepos.
        self.header_lst = list(map(lambda x: self.header.split(delimiter)[x], self.__valuepos)) if self.__valuepos else self.header_lst

    def _read_body(self,keypos,delimiter,keycast,cast):
        for lineno, row in enumerate(self._fp):
            row = row.rstrip()
            atoms = row.split(delimiter)
            # thiscols: Number of columns of current line.
            thiscols = len(atoms)
            if thiscols < self.ncols:
                self._short_line_handle(row,lineno,self.ncols)    
                continue

            key = atoms[keypos]
            value = list(map(lambda x:atoms[x], self.__valuepos)) if (self.__valuepos is not None) else atoms
            if len(value) == 1:
                value = value[0]
            if keycast:
                key = keycast(key)
            if cast:
                value = cast(value)
            self[key] = value

        assert thiscols, "File empty"
        self.ncols = thiscols
        logging.debug("Imported {0} records from `{1}`.".\
                    format(len(self), self.filename))

    def _short_line_handle(self,row,lineno,ncols):
        action = "Aborted" if self.strict else "Skipped"

        msg = "Must contain >= {0} columns.  {1}.\n".format(ncols, action)
        msg += "  --> Line {0}: {1}".format(lineno + 1, row)
        logging.error(msg)
        if self.strict:
            sys.exit(1)

    #Convert single item to list.
    def __value2list(self, value):
        #if not self.parse_value:
        #    return value
        if isinstance(value, map):
            value = list(value)
        if isinstance(value, list):
            return value
        ### Why return None
        if value == None:
            return value
            #return [0]
        return [value]

    #get length of value list[Abord]
    def valueLength(self):
        if self.__valuepos == None:
            return self.ncols
        else:
            return len(self.__valuepos)

    def max(self, values):
        if values == None:
            return values
        return max(values)

class SetFile(BaseFile, set):
    """ Read File into set."""
    def __init__(self, filename, valuepos=0, delimiter=False,
                       keycast=None, cast=None, has_header=False):
        super(BaseFile, self).__init__()

        ## Load attribute.
        self._fp = must_open(filename)
        self.filename = filename
        ## Should move this two lines to BaseFile?
        self.has_header = has_header
        #self.valuepos = valuepos
        #if has_header:
        #    self._read_header(delimiter)
        for line in self._fp:
            la = line.strip()
            # Escape from empty lines.
            if len(la) == 0:
                continue
            if delimiter is False:
                item = la
            else:
                la = la.split(delimiter)
                item = la[valuepos]
            self.add(item)

class CaseInsensitiveSet(set):
    """Make sense when element is tuple
    Refers to https://stackoverflow.com/questions/27531211/how-to-get-case-insensitive-python-set"""

    def get_lower(self, item):
        """Get lower for string or tuple"""
        if isinstance(item, str):
            return item.lower()
        elif isinstance(item, tuple):
            return tuple([self.get_lower(i) for i in item])
        else:
            return item

    def add(self, item):
        set.add(self, self.get_lower(item))

    def __contains__(self, item):
        return set.__contains__(self, self.get_lower(item))

def must_open(filename, mode="r", checkexists=False, skipcheck=False, \
            oappend=False):
    """
    Accepts filename and returns filehandle.

    Checks on multiple files, stdin/stdout/stderr, .gz or .bz2 file.
    """
    import io
    if isinstance(filename, io.IOBase):
        return filename

    if isinstance(filename, list):
        assert "r" in mode

        if filename[0].endswith(".gz") or filename[0].endswith(".bz2"):
            filename = " ".join(filename)  # allow opening multiple gz/bz2 files
        else:
            import fileinput
            return fileinput.input(filename)

    #if filename.startswith("s3://"):
    #    from jcvi.utils.aws import pull_from_s3
    #    filename = pull_from_s3(filename)

    if filename in ("-", "stdin"):
        assert "r" in mode
        fp = sys.stdin

    elif filename == "stdout":
        assert "w" in mode
        fp = sys.stdout

    elif filename == "stderr":
        assert "w" in mode
        fp = sys.stderr

    elif filename == "tmp" and mode == "w":
        from tempfile import NamedTemporaryFile
        fp = NamedTemporaryFile(delete=False)

    elif filename.endswith(".gz"):
        if 'r' in mode:
            cmd = "zcat {0}".format(filename)
            fp = popen(cmd, debug=False)
        elif 'w' in mode:
            import gzip
            fp = gzip.open(filename, mode)

    elif filename.endswith(".bz2"):
        if 'r' in mode:
            cmd = "bzcat {0}".format(filename)
            fp = popen(cmd, debug=False)
        elif 'w' in mode:
            import bz2
            fp = bz2.BZ2File(filename, mode)
    else:
        if checkexists:
            assert mode == "w"
            overwrite = (not op.exists(filename)) if skipcheck \
                        else check_exists(filename, oappend)
            if overwrite:
                if oappend:
                    fp = open(filename, "a")
                else:
                    fp = open(filename, "w")
            else:
                logging.debug("File `{0}` already exists. Skipped."\
                        .format(filename))
                return None
        else:
            fp = open(filename, mode)

    return fp

def grouped(iterable, n):
    "s -> (s0,s1,s2,...sn-1), (sn,sn+1,sn+2,...s2n-1), (s2n,s2n+1,s2n+2,...s3n-1), ..."
    return zip(*[iter(iterable)]*n)

def if_str_eq(s1, s2, ignorecase=False):
    """ Test if two strings are the same. 
    Return the results of s1 == s2.
    """
    assert isinstance(s1, str)
    assert isinstance(s2, str)
    if ignorecase:
        return s1.lower() == s2.lower()
    else:
        return s1 == s2

def pairwise(iterable):
    return grouped(iterable, 2)

if __name__ == '__main__':

    print("hhahah")
